-------------------------------------------------------------------------
--          PAQUET DE PROCEDIMENTS DE CONSULTA
-- Tots els procediments tenen un parmetre de sortida que es una variable
-- REF CURSOR. Aquesta variable s'utilitzara per retornar la consulta demanada
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE Consultes IS

  TYPE t_RefCur IS REF CURSOR;

  PROCEDURE Desenvolupadors_Pais(
    p_Pais IN descarrega.pais%TYPE,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Aplicacions_Actives(
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Paisos_Descarrega(
    p_aplicacio IN aplicacio.idaplicacio%TYPE,
    p_any IN NUMBER,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Activitat_Usuari(
    p_telefon IN usuari.numeromobil%TYPE,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Usuaris_DinersGastats(
    p_any IN NUMBER,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2);
    
END Consultes;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY Consultes IS

  v_log logsprocediments.parametresentrada%type;
  
------------------------------------------------------------------------------------ 
--     CONSULTA DE DESENVOLUPADORS D'UN PAS DONAT
-- Retorna un cursor apuntant a la llista de desenvolupadors d'una pas donat, 
-- amb totes les seves dades, incloent el nmero d'aplicacions diferents publicades.
------------------------------------------------------------------------------------
  PROCEDURE Desenvolupadors_Pais(
    p_Pais IN descarrega.pais%TYPE,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      -------  Obertura del cursor sobre el select de la consulta -----------------
      OPEN p_cursor FOR 
        SELECT pe.nom, pe.cognoms, de.codidesenvolupador, de.empresa, count(*) as numapps, pe.adrea, pe.poblacio, pe.telefon, pe.email 
        FROM (desenvolupador de JOIN persona pe ON de.iddesenvolupador=pe.idpersona) 
              JOIN apldesenvolupada ap ON ap.desenvolupador=de.iddesenvolupador
        WHERE pe.pais=p_Pais
        GROUP BY pe.nom, pe.cognoms, pe.adrea, pe.poblacio, pe.telefon, pe.email, de.codidesenvolupador, de.empresa;
                  
      rsp:='OK'; 
    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Pais=' || p_Pais);  
    rsp:='Dades dels desenvolupadors del pais: ' || p_Pais || ' i num. d''apps. desenvolupades.  RSP:' || rsp;
    Util.InsertarLog('Desenvolupadors_Pais', v_log, rsp); 
  
  END Desenvolupadors_Pais;

------------------------------------------------------------------------------------ 
--      CONSULTA D'APLICACIONS ACTIVES
-- Retorna un cursor apuntant a la llista de totes les aplicacions actives i de les
-- seves dades principals, en ordre descendent pel nmero total de descrregues.
------------------------------------------------------------------------------------
  PROCEDURE Aplicacions_Actives(
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      -------  Obertura del cursor sobre el select de la consulta -----------------
      OPEN p_cursor FOR 
        SELECT ap.idaplicacio, ap.versio,  count(*) as numDescarregues, ap.datapujada, ap.resoluciomin, ap.url
        FROM aplicacio ap JOIN descarrega de ON ap.idaplicacio=de.aplicacio
        WHERE ap.activa=1
        GROUP BY ap.idaplicacio, ap.versio, ap.datapujada, ap.url, ap.resoluciomin
        ORDER BY numDescarregues DESC;
                  
      rsp:='OK'; 
    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Cap parmetre d''entrada');  
    rsp:='Llista de les dades principals d''aplicacions activas.   RSP:' || rsp;
    Util.InsertarLog('Aplicacions_Actives', v_log, rsp); 
    

  END Aplicacions_Actives;
  
------------------------------------------------------------------------------------ 
--     CONSULTA DELS PASOS ON S'HA DESCARREGAT UNA APLICACIO EN UN ANY CONCRET
-- Retorna un cursor apuntant a la llista de pasos on s'ha descarregat l'aplicaci 
-- l'any donat, aix com el nmero de descrregues que ha tingut a cada pas.
------------------------------------------------------------------------------------  
  PROCEDURE Paisos_Descarrega(
    p_aplicacio IN aplicacio.idaplicacio%TYPE,
    p_any IN NUMBER,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      -------  Obertura del cursor sobre el select de la consulta -----------------
      OPEN p_cursor FOR 
       SELECT pa.nom, count(*) as numDescarregues
        FROM (aplicacio ap JOIN descarrega de ON ap.idaplicacio=de.aplicacio)
            JOIN pais pa ON de.pais=pa.idpais
        WHERE EXTRACT(YEAR FROM de.datadescarrega)=p_any AND ap.idaplicacio=p_aplicacio 
        GROUP BY pa.nom;
                  
      rsp:='OK'; 
    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Aplicacio=' || p_aplicacio ||', Any=' || p_any );  
    rsp:='Llista de pasos on s''ha descarregat l''aplicacio en l''any donat.   RSP:' || rsp;
    Util.InsertarLog('Paisos_Descarrega', v_log, rsp); 
    

  END Paisos_Descarrega;    

------------------------------------------------------------------------------------ 
--     CONSULTA DE LA ACTIVITAT DE DESCRREGUES D'UN USUARI
-- Retorna un cursor apuntant a la llista de dades de descrregues efectuades 
-- per l'usuari propietari del nmero mbil passat com a parmetre. Inclou tots
-- els dispositius del usuari amb aquest nmero mbil.
------------------------------------------------------------------------------------ 
  PROCEDURE Activitat_Usuari(
    p_telefon IN usuari.numeromobil%TYPE,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      -------  Obertura del cursor sobre el select de la consulta -----------------
      OPEN p_cursor FOR 
        SELECT aplicacio, datadescarrega, imei, preu, modepagament
        FROM (descarrega NATURAL JOIN dispositiu) 
        WHERE imei IN
            (SELECT di.imei 
             FROM dispositiu di JOIN usuari us ON di.idpropietari=us.idusuari
             WHERE us.numeroMobil=p_telefon)
        ORDER BY datadescarrega;
                  
      rsp:='OK'; 
    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Nmero Mbil=' || p_telefon);  
    rsp:='Descrregues de l''usuari amb mbil: ' || p_telefon || '.  RSP:' || rsp;
    Util.InsertarLog('Activitat_Usuari', v_log, rsp);   
  
  
  END Activitat_Usuari;
  
------------------------------------------------------------------------------------ 
--        CONSULTA DELS USUARIS QUE MES DINERS S'HAN GASTAT EN APLICACIONS
-- Retorna un cursor apuntant a la llista dels 20 usuaris que mes diners s'han gastat
-- en aplicacions en l'any donat com a parmetre d'entrada.
------------------------------------------------------------------------------------   
  PROCEDURE Usuaris_DinersGastats(
    p_any IN NUMBER,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      -------  Obertura del cursor sobre el select de la consulta -----------------
      OPEN p_cursor FOR 
        SELECT nom, cognoms, diners_gastats FROM
          (SELECT  idpersona, nom, cognoms, sum(preu) as diners_gastats
           FROM  (descarrega NATURAL JOIN dispositiu) JOIN persona ON idpropietari=idpersona
           WHERE EXTRACT(YEAR FROM datadescarrega)=p_any
           GROUP BY idpersona, nom, cognoms
           ORDER BY diners_gastats DESC)
        WHERE ROWNUM < 21;
                  
      rsp:='OK'; 
    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Any=' || p_any );  
    rsp:='Llista dels 20 usuaris que mes diners han gastat en l''any donat.   RSP:' || rsp;
    Util.InsertarLog('Usuaris_DinersGastats', v_log, rsp); 

    
  END Usuaris_DinersGastats;

END Consultes;
/