-------------------------------------------------------------------------
--      PAQUET DE MANTENIMENT D'APLICACIONS I ENTITATS ASSOCIADES
-- Els procediments d'alta i modificaci s'engloban en un de sol,
-- seleccionant la opci corresponent amb el primer parmetre d'entrada.
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE MantenimentAplicacions IS

  PROCEDURE AM_Aplicacio(
    p_Opcio IN CHAR,
    p_Identificador IN aplicacio.idaplicacio%TYPE,
    p_Versio IN aplicacio.versio%TYPE,
    p_Data IN aplicacio.datapujada%TYPE,
    p_Url IN aplicacio.url%TYPE,
    p_Resolucio IN aplicacio.resoluciomin%TYPE,
    p_Activa IN aplicacio.activa%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Aplicacio(
    p_Identificador IN aplicacio.idaplicacio%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE AM_SistSuportat(
    p_opcio IN CHAR,
    p_Aplicacio IN sistsuportat.aplicacio%TYPE,
    p_SistOperatiu IN sistsuportat.sistemaoperatiu%TYPE,
    p_MidaAplicacio IN sistsuportat.midaaplicacio%TYPE,
    p_FitxerBinari IN sistsuportat.fitxerbinari%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_SistSuportat(
    p_Aplicacio IN sistsuportat.aplicacio%TYPE,
    p_SistOperatiu IN sistsuportat.sistemaoperatiu%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE AM_Preu(
    p_opcio IN CHAR,
    p_Aplicacio IN aplpreu.aplicacio%TYPE,
    p_Pais IN aplpreu.pais%TYPE,
    p_Preu IN aplpreu.preu%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Preu(
    p_Aplicacio IN aplpreu.aplicacio%TYPE,
    p_Pais IN aplpreu.pais%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE AM_Descripcio(
    p_opcio IN CHAR,
    p_Aplicacio IN apldescrita.aplicacio%TYPE,
    p_Idioma IN apldescrita.idioma%TYPE,
    p_Descripcio IN apldescrita.descripcio%TYPE,
    rsp OUT NOCOPY VARCHAR2);
  
  PROCEDURE Baixa_Descripcio(
    p_Aplicacio IN apldescrita.aplicacio%TYPE,
    p_Idioma IN apldescrita.idioma%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE AB_Desenvolupada(
    p_opcio IN CHAR,
    p_Aplicacio IN apldesenvolupada.aplicacio%TYPE,
    p_Desenvolupador IN apldesenvolupada.desenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
END MantenimentAplicacions;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY MantenimentAplicacions IS

  v_log logsprocediments.parametresentrada%type;
  v_log_opcio VARCHAR2(12);
  resHoritzontal VARCHAR2(10);
  resVertical VARCHAR2(10);
  OPCIO_INCORRECTE EXCEPTION;
  NO_TROBAT EXCEPTION;
  FORMAT_RESOLUCIO_INCORRECTE EXCEPTION;
  
--------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions d'aplicacions
--------------------------------------------------------------------------------
  PROCEDURE AM_Aplicacio(
    p_Opcio IN CHAR,
    p_Identificador IN aplicacio.idaplicacio%TYPE,
    p_Versio IN aplicacio.versio%TYPE,
    p_Data IN aplicacio.datapujada%TYPE,
    p_Url IN aplicacio.url%TYPE,
    p_Resolucio IN aplicacio.resoluciomin%TYPE,
    p_Activa IN aplicacio.activa%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      resHoritzontal:=substr(p_Resolucio,1,instr(p_Resolucio,'x')-1);
      resVertical:= substr(p_Resolucio,instr(p_Resolucio,'x')+1,length(p_Resolucio)-instr(p_Resolucio,'x'));
       IF resHoritzontal IS NULL OR resVertical IS NULL THEN  
          RAISE FORMAT_RESOLUCIO_INCORRECTE;
      ELSE
          resHoritzontal:=translate(resHoritzontal,'T0123456789','T');
          resVertical:=translate(resVertical,'T0123456789','T');
          IF resHoritzontal IS NOT NULL OR resVertical IS NOT NULL THEN 
                RAISE FORMAT_RESOLUCIO_INCORRECTE;
          END IF;
      END IF;

      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO APLICACIO (idaplicacio, versio, datapujada, url, resoluciomin, activa) 
            VALUES (p_identificador, p_versio, p_data, p_url, p_resolucio, p_activa);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE APLICACIO SET versio=p_versio, datapujada=p_data, url=p_url, 
                resoluciomin=p_resolucio, activa=p_activa
                WHERE idaplicacio=p_identificador;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: L''aplicaci ja existeix');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''aplicaci no existeix.');
          ROLLBACK;
      WHEN FORMAT_RESOLUCIO_INCORRECTE THEN
          rsp:= ('ERROR: El format de resoluci no es correcte. Ha de ser ''DIGITSxDIGITS''- Exemple: 480x800');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Opci=' || p_opcio ||', IdAp.=' || p_identificador || ', Vers.='|| p_versio || ', Data=' || p_data || 
            ', Url=' || p_url || ', Res.Min=' || p_resolucio || ', Activa=' ||  p_activa); 
    Util.InsertarLog('AM_Aplicacio:' || v_log_opcio, v_log, rsp);
  
  END AM_Aplicacio;

---------------------------------------------------------------------  
-- Procediment de baixas d'aplicacions
-------------------------------------------------------------------    
  PROCEDURE Baixa_Aplicacio(
    p_Identificador IN aplicacio.idaplicacio%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM APLICACIO WHERE idaplicacio=p_identificador;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''aplicaci no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: S''han fet descrregues d''aquesta aplicaci. No es pot donar de baixa.';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
    END;
   
    -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdAp.=' || p_identificador);
    Util.InsertarLog('Baixa_Aplicacio', v_log, rsp);
    
  END Baixa_Aplicacio;

--------------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions de sistemes operatius suportats per aplicacions
--------------------------------------------------------------------------------------
  PROCEDURE AM_SistSuportat(
    p_opcio IN CHAR,
    p_Aplicacio IN sistsuportat.aplicacio%TYPE,
    p_SistOperatiu IN sistsuportat.sistemaoperatiu%TYPE,
    p_MidaAplicacio IN sistsuportat.midaaplicacio%TYPE,
    p_FitxerBinari IN sistsuportat.fitxerbinari%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO SISTSUPORTAT (aplicacio, sistemaoperatiu, midaaplicacio, fitxerbinari) 
            VALUES (p_Aplicacio, p_SistOperatiu, p_MidaAplicacio, p_FitxerBinari);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE SISTSUPORTAT SET midaaplicacio=p_MidaAplicacio, fitxerbinari=p_FitxerBinari
                WHERE aplicacio=p_Aplicacio AND sistemaoperatiu=p_SistOperatiu;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: L''aplicaci ja suporta aquest sistema operatiu');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''aplicaci i el sist. operatiu no estan relacionats o no existeixen.');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2291 THEN
            rsp:='ERROR: L''aplicacio i/o el sist. operatiu no existeixen';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments
      -------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdAp.=' || p_Aplicacio || ', SistOp.=' || p_SistOperatiu || 
              ', Mida=' ||  p_MidaAplicacio || ', Fitxer=' || p_FitxerBinari); 
      Util.InsertarLog('AM_SistSuportat:' || v_log_opcio, v_log, rsp);

  END AM_SistSuportat;
    
--------------------------------------------------------------------------------  
-- Procediment de baixa de sistemes operatius suportats per aplicacions
--------------------------------------------------------------------------------
  PROCEDURE Baixa_SistSuportat(
    p_Aplicacio IN sistsuportat.aplicacio%TYPE,
    p_SistOperatiu IN sistsuportat.sistemaoperatiu%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      DELETE FROM SISTSUPORTAT WHERE aplicacio=p_Aplicacio AND sistemaoperatiu=p_SistOperatiu;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:='ERROR: L''aplicaci i el sist. operatiu no estan relacionats o no existeixen';
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;
  
      -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdAp.=' || p_Aplicacio || ', SistOp.=' || p_SistOperatiu);
    Util.InsertarLog('Baixa_SistSuportat', v_log, rsp);  
  
  END Baixa_SistSuportat;  
    
--------------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions dels preus de les aplicacions
--------------------------------------------------------------------------------------
  PROCEDURE AM_Preu(
    p_opcio IN CHAR,
    p_Aplicacio IN aplpreu.aplicacio%TYPE,
    p_Pais IN aplpreu.pais%TYPE,
    p_Preu IN aplpreu.preu%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO APLPREU (aplicacio, pais, preu) 
            VALUES (p_Aplicacio, p_Pais, p_Preu);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE APLPREU SET preu=p_Preu
                WHERE aplicacio=p_Aplicacio AND pais=p_Pais;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: L''aplicaci ja t un preu fixat per aquest pas');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''aplicaci no te un preu per aquest pas o no existeix el pas i/o l''aplicaci');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2291 THEN
            rsp:='ERROR: L''aplicaci i/o el pas no existeixen';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments
      -------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdAp.=' || p_Aplicacio || ', Pais=' || p_Pais || 
              ', Preu=' ||  p_Preu); 
      Util.InsertarLog('AM_Preu:' || v_log_opcio, v_log, rsp);

  END AM_Preu;

--------------------------------------------------------------------------------------  
-- Procediment de baixa de preus de les aplicacions
--------------------------------------------------------------------------------------
  PROCEDURE Baixa_Preu(
    p_Aplicacio IN aplpreu.aplicacio%TYPE,
    p_Pais IN aplpreu.pais%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      DELETE FROM APLPREU WHERE aplicacio=p_Aplicacio AND pais=p_Pais;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:='ERROR: L''aplicaci no te un preu per aquest pas o no existeix el pas i/o l''aplicaci';
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;
  
      -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdAp.=' || p_Aplicacio || ', Pais=' || p_Pais);
    Util.InsertarLog('Baixa_Preu', v_log, rsp);  
  
  END Baixa_Preu;  

--------------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions de les descripcions d'aplicacions
--------------------------------------------------------------------------------------
  PROCEDURE AM_Descripcio(
    p_opcio IN CHAR,
    p_Aplicacio IN apldescrita.aplicacio%TYPE,
    p_Idioma IN apldescrita.idioma%TYPE,
    p_Descripcio IN apldescrita.descripcio%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO APLDESCRITA (aplicacio, idioma, descripcio) 
            VALUES (p_Aplicacio, p_Idioma, p_Descripcio);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE APLDESCRITA SET descripcio=p_Descripcio
                WHERE aplicacio=p_Aplicacio AND idioma=p_idioma;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: L''aplicaci ja est descrita en aquest idioma');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''aplicaci no est descrita en aquest idioma o no existeix l''idioma i/o l''aplicaci');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2291 THEN
            rsp:='ERROR: L''aplicaci i/o l''idioma no existeixen';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments. 
      -- El parmetre d'entrada 'p_Descripci es trunca a les primeres 30 posicions
      -- ja que en cas contrari podria arrivar fins a 1024 bytes i no es pot emmagatzemar-ho tot.
      ---------------------------------------------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdAp.=' || p_Aplicacio || ', Idioma=' || p_Idioma || 
              ', Descripcio=' ||  SUBSTR(p_Descripcio,1,30) || '...'); 
      Util.InsertarLog('AM_Descripcio:' || v_log_opcio, v_log, rsp);

  END AM_Descripcio;

 --------------------------------------------------------------------------------------  
-- Procediment de baixa de descripci de les aplicacions
-------------------------------------------------------------------------------------- 
  PROCEDURE Baixa_Descripcio(
    p_Aplicacio IN apldescrita.aplicacio%TYPE,
    p_Idioma IN apldescrita.idioma%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM APLDESCRITA WHERE aplicacio=p_Aplicacio AND idioma=p_Idioma;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:='ERROR: L''aplicaci no est descrita en aquest idioma o no existeix l''idioma i/o l''aplicaci';
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;
  
      -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdAp.=' || p_Aplicacio || ', Idioma=' || p_Idioma);
    Util.InsertarLog('Baixa_Descripcio', v_log, rsp);  
  
  END Baixa_Descripcio;  

--------------------------------------------------------------------------------------  
-- Procediment d'altes i baixes dels desenvolupadors d'una aplicaci.
-- No es considera la possibilitat de modificaci ja que aquesta
-- entitat no t cap atribut, llevat dels que configuran la clau primaria.
--------------------------------------------------------------------------------------
  PROCEDURE AB_Desenvolupada(
    p_opcio IN CHAR,
    p_Aplicacio IN apldesenvolupada.aplicacio%TYPE,
    p_Desenvolupador IN apldesenvolupada.desenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO APLDESENVOLUPADA (aplicacio, desenvolupador) 
            VALUES (p_Aplicacio, p_Desenvolupador);   

        WHEN 'B' THEN
            v_log_opcio:='Baixa';
            DELETE FROM APLDESENVOLUPADA WHERE aplicacio=p_Aplicacio 
                    AND desenvolupador=p_Desenvolupador;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: Aquesta persona ja est registrada com a desenvolupador d''aquesta aplicaci');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o B=Baixa');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: Aquesta persona no ha desenvolupat l''aplicaci o no existeix el desenvolupador i/o l''aplicaci');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2291 THEN
            rsp:='ERROR: L''aplicaci i/o el desenvolupador no existeixen';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments. 
      ---------------------------------------------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdAp.=' || p_Aplicacio || ', Desenvolupador=' || p_Desenvolupador); 
      Util.InsertarLog('AB_Desenvolupada:' || v_log_opcio, v_log, rsp);

  END AB_Desenvolupada;
  
    
END MantenimentAplicacions;
/