-------------------------------------------------------------------------
--  PAQUET DE MANTENIMENT DE LES DESCARREGES REATLITZADES PELS USUARIS
--
--    Es crean dos procediments d'alta sobrecarregats:  Un a partir dels tres
--  parmetres indispensables per poder emmagatzemar la informaci: 
--  L'Imei del dispositiu de descrrega, l'aplicaci descarregada i el mode de pagament.
--  La resta de dades associades s'extreuen de la mateixa base de dades.
--
--    L'altre procediment d'alta,  permet introduir totes les dades d'una 
--  descrrega en cas de que es necesiti emmagatzemar una informaci diferent a la 
--  que hi ha en aquell moment a la base de dades. Per exemple una data diferent 
--  de la del sistema, un preu diferent al referenciat en la taula APLPREU o un
--  operador o sistema operatiu diferent al que te l'usuari en aquell moment.
--
--    Tamb es crea un proces de modificaci i un altre de baixa en base al 
--  identicador de la descrrega. Aquest identificador s'ha retornat com a
--  parmetre de sortida en el moment de l'alta.
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE MantenimentDescarregues IS

  PROCEDURE Alta(
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    p_Identificador OUT NOCOPY descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Alta(
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_Pais IN descarrega.pais%TYPE,
    p_NomOperador IN descarrega.nomoperador%TYPE,
    p_Data IN descarrega.datadescarrega%TYPE,
    p_Preu IN descarrega.preu%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    p_Identificador OUT NOCOPY descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Modificacio(
    p_Identificador IN descarrega.iddescarrega%TYPE,
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_Pais IN descarrega.pais%TYPE,
    p_NomOperador IN descarrega.nomoperador%TYPE,
    p_Data IN descarrega.datadescarrega%TYPE,
    p_Preu IN descarrega.preu%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa(
    p_Identificador IN descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2);
   
END MantenimentDescarregues;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY MantenimentDescarregues IS

  v_logEnt logsprocediments.parametresentrada%type;
  v_logSor logsprocediments.parametresentrada%type;
  APL_PREU_INCORRECTE EXCEPTION;
  NO_TROBAT EXCEPTION;
  SIST_SUPORTAT_INCORRECTE EXCEPTION;
  APLICACIO_NO_ACTIVA EXCEPTION;
  RESOLUCIO_NO_SUPORTADA EXCEPTION;
  
--------------------------------------------------------------------------------  
-- Procediment d'alta d'una descrrega amb valors existents a la base de dades.
--------------------------------------------------------------------------------
  PROCEDURE Alta(
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    p_Identificador OUT NOCOPY descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

    -- Declaracio de variables
    ----------------------------------------------------------------------------
    v_Pais descarrega.pais%TYPE;
    v_NomOperador descarrega.nomoperador%TYPE;
    v_Preu descarrega.preu%TYPE;
    v_Usuari dispositiu.idPropietari%TYPE;
    v_SisOperatiu dispositiu.sodispositiu%TYPE;
    v_resolucio_disp dispositiu.resolucio%TYPE;
    v_resolucio_app aplicacio.resoluciomin%TYPE;
    v_contador NUMBER;
    
    v_res_horitzontal_dis NUMBER;
    v_res_vertical_dis NUMBER;
    v_res_horitzontal_app NUMBER;
    v_res_vertical_app NUMBER;

  BEGIN
    BEGIN
 
      SELECT idpropietari, sodispositiu, resolucio INTO v_Usuari, v_SisOperatiu, v_resolucio_disp 
      FROM DISPOSITIU WHERE imei=p_Imei;

     -- Control per veure si l'aplicaci suporta el sistema operatiu del dispositiu
      SELECT COUNT(*) INTO v_contador FROM SISTSUPORTAT WHERE aplicacio=p_Aplicacio AND sistemaoperatiu=v_SisOperatiu;
      IF v_contador = 0 THEN  RAISE SIST_SUPORTAT_INCORRECTE;
      END IF;
      
      SELECT operadorusuari INTO v_NomOperador FROM USUARI  WHERE idusuari=v_Usuari;
      SELECT pais INTO v_Pais FROM PERSONA WHERE idpersona=v_Usuari;

      -- Control per veure si l'aplicaci te un preu associat al pais de registre del usuari   
      SELECT COUNT(*) INTO v_contador FROM APLPREU WHERE aplicacio=p_Aplicacio AND pais=v_Pais;
      IF v_contador = 0 THEN RAISE APL_PREU_INCORRECTE;
      END IF;
      
      -- Control per comprovar si l'aplicaci esta activa per a descrrega      
      SELECT activa, resoluciomin INTO v_contador, v_resolucio_app FROM APLICACIO WHERE idaplicacio=p_aplicacio;
      IF v_contador = 0 THEN RAISE APLICACIO_NO_ACTIVA;
      END IF;
            
      -- Control per comprovar que el dispositiu suporti la resoluci de l'aplicaci.
      v_res_horitzontal_dis:=substr(v_resolucio_disp,1,instr(v_resolucio_disp,'x')-1);
      v_res_vertical_dis:=substr(v_resolucio_disp,instr(v_resolucio_disp,'x')+1,length(v_resolucio_disp)-instr(v_resolucio_disp,'x'));
      v_res_horitzontal_app:=substr(v_resolucio_app,1,instr(v_resolucio_app,'x')-1);
      v_res_vertical_app:=substr(v_resolucio_app,instr(v_resolucio_app,'x')+1,length(v_resolucio_app)-instr(v_resolucio_app,'x'));
  
      IF v_res_horitzontal_dis < v_res_horitzontal_app OR v_res_vertical_dis < v_res_vertical_app THEN
        IF v_res_horitzontal_dis < v_res_vertical_app OR v_res_vertical_dis < v_res_horitzontal_app
          THEN RAISE RESOLUCIO_NO_SUPORTADA;
        END IF;
      END IF;

      SELECT preu INTO v_preu FROM APLPREU WHERE aplicacio=p_Aplicacio AND pais=v_Pais;
           
      INSERT INTO DESCARREGA (aplicacio, imei, pais, nomoperador, datadescarrega, preu, modepagament) 
        VALUES (p_Aplicacio, p_Imei, v_Pais, v_NomOperador, SYSDATE, v_Preu, p_ModePagament)
        RETURNING iddescarrega INTO p_Identificador; 

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN APL_PREU_INCORRECTE THEN
          rsp:= ('ERROR: L''aplicaci no existeix o no te un preu associat al pais de registre del usuari');
          ROLLBACK;
      WHEN SIST_SUPORTAT_INCORRECTE THEN
          rsp:= ('ERROR: L''aplicaci no existeix o no suporta el sistema operatiu del dispositiu de descrrega');
          ROLLBACK;    
      WHEN NO_DATA_FOUND THEN
          rsp:= ('ERROR: El dispositiu de descrrega no existeix.');
          ROLLBACK;
      WHEN APLICACIO_NO_ACTIVA THEN
          rsp:= ('ERROR: L''aplicaci no esta activa per a descrrega.');
          ROLLBACK;
      WHEN RESOLUCIO_NO_SUPORTADA THEN
          rsp:= ('ERROR: El dispositiu no suporta la resoluci mnima de l''aplicaci');
          ROLLBACK;       
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_logEnt:=('Aplicaci=' || p_Aplicacio ||', Imei=' || p_Imei || ', Mode Pag.='|| p_ModePagament); 
    v_logSor:=('IdDescarrega=' || p_Identificador || ', Resposta= ' || rsp);
    Util.InsertarLog('Alta Descrrega', v_logEnt, v_logSor );
  
  END Alta;
  
--------------------------------------------------------------------------------  
-- Procediment d'alta manual d'una descrrega.
-- No es tenen en compte el valors actuals del usuari en quan a operador,
-- sistema operatiu o preu de l'aplicaci en  el pas de registre del usuari. 
-- Es comproven per les restriccions referencials corresponents dels atributs
-- Aplicacio, Imei, Pas i Operador
--------------------------------------------------------------------------------  
  PROCEDURE Alta(
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_Pais IN descarrega.pais%TYPE,
    p_NomOperador IN descarrega.nomoperador%TYPE,
    p_Data IN descarrega.datadescarrega%TYPE,
    p_Preu IN descarrega.preu%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    p_Identificador OUT NOCOPY descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      INSERT INTO DESCARREGA (aplicacio, imei, pais, nomoperador, datadescarrega, preu, modepagament) 
        VALUES (p_Aplicacio, p_Imei, p_Pais, p_NomOperador, p_Data, p_Preu, p_ModePagament)
        RETURNING iddescarrega INTO p_Identificador; 

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_logEnt:=('Apl=' || p_Aplicacio ||', Imei=' || p_Imei || ', Pas='|| p_Pais ||', Operador='|| p_NomOperador ||
               ', Data='|| p_Data || ', Preu='|| p_Preu ||', M.Pag.='|| p_ModePagament);  
    v_logSor:=('IdDescarrega=' || p_Identificador || ', Resposta= ' || rsp);
    Util.InsertarLog('Alta Descrrega', v_logEnt, v_logSor );
  
  END Alta;    

--------------------------------------------------------------------------------------  
-- Procediment de modificaci de descrregues fetes pels usuaris.
-- Igual que en el procediment d'alta anterior, ne es t en compte el valors 
-- actuals del usuari en quan a operador, sistema operatiu o preu de l'aplicaci
-- en  el pas de registre del usuari. 
-- Es comproven igualment les restriccions referencials corresponents dels atributs
-- Aplicacio, Imei, Pas i Operador.
--------------------------------------------------------------------------------------
  PROCEDURE Modificacio(
    p_Identificador IN descarrega.iddescarrega%TYPE,
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_Pais IN descarrega.pais%TYPE,
    p_NomOperador IN descarrega.nomoperador%TYPE,
    p_Data IN descarrega.datadescarrega%TYPE,
    p_Preu IN descarrega.preu%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      UPDATE DESCARREGA SET aplicacio=p_Aplicacio, imei=p_Imei, pais=p_Pais, nomoperador=p_NomOperador,
                datadescarrega=p_Data, preu=p_Preu, modepagament=p_ModePagament
        WHERE iddescarrega=p_Identificador;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: La descrrega no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments
      -------------------------------------------------------
      v_logEnt:=('IdDesc.=' || p_Identificador || ', Aplicaci=' ||  p_Aplicacio ||
              ', Imei=' || p_Imei || ', Pas=' || p_Pais || ', Operador=' ||  p_NomOperador ||
              ', DataDesc.=' || p_Data || ', Preu=' || p_Preu || ', ModePag.=' || p_ModePagament); 
      Util.InsertarLog('Modificacio', v_logEnt, rsp);

  END Modificacio;

---------------------------------------------------------------------  
-- Procediment de baixa de descrregues fetes pels usuaris
-------------------------------------------------------------------    
  PROCEDURE Baixa(
    p_Identificador IN descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM DESCARREGA WHERE iddescarrega=p_Identificador;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: La descrrega no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;
   
    -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_logEnt:=('Identificador Desc.=' || p_Identificador);
    Util.InsertarLog('Baixa Descrrega', v_logEnt, rsp);
    
  END Baixa;
    
END MantenimentDescarregues;
/