-------------------------------------------------------------------------
--  PAQUET DE MANTENIMENT DELS DESENVOLUPADORS I ENTITATS ASSOCIADES
-- Els procediments d'alta i modificaci s'engloban en un de sol,
-- seleccionant la opci corresponent amb el primer parmetre d'entrada.
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE MantenimentDesenvolupadors IS

  PROCEDURE AM_Empresa(
    p_Opcio IN CHAR,
    p_IdEmpresa IN empresa.idempresa%TYPE,
    p_PaisRegistre IN empresa.paisregistre%TYPE,
    p_Representant IN empresa.representant%TYPE,
    p_Adrea IN empresa.adrea%TYPE,
    p_Poblacio IN empresa.poblacio%TYPE,
    p_Telefon IN empresa.telefon%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Empresa(
    p_IdEmpresa IN empresa.idempresa%TYPE,
    rsp OUT NOCOPY VARCHAR2);
   
  PROCEDURE AM_Desenvolupador(
    p_opcio IN CHAR,
    p_IdDesenvolupador IN desenvolupador.iddesenvolupador%TYPE,
    p_Empresa IN desenvolupador.empresa%TYPE,
    p_CodiDesenvolupador IN desenvolupador.codidesenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Desenvolupador(
    p_IdDesenvolupador IN desenvolupador.iddesenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2);
   
END MantenimentDesenvolupadors;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY MantenimentDesenvolupadors IS

  v_log logsprocediments.parametresentrada%type;
  v_log_opcio VARCHAR2(12);
  OPCIO_INCORRECTE EXCEPTION;
  NO_TROBAT EXCEPTION;
  
--------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions d'empreses
--------------------------------------------------------------------------------
  PROCEDURE AM_Empresa(
    p_Opcio IN CHAR,
    p_IdEmpresa IN empresa.idempresa%TYPE,
    p_PaisRegistre IN empresa.paisregistre%TYPE,
    p_Representant IN empresa.representant%TYPE,
    p_Adrea IN empresa.adrea%TYPE,
    p_Poblacio IN empresa.poblacio%TYPE,
    p_Telefon IN empresa.telefon%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO EMPRESA (idempresa, paisregistre, representant, adrea, poblacio, telefon) 
            VALUES (p_IdEmpresa, p_PaisRegistre, p_Representant, p_Adrea, p_Poblacio, p_Telefon);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE EMPRESA SET paisregistre=p_PaisRegistre, representant=p_Representant, 
                adrea=p_Adrea, telefon=p_Telefon
                WHERE idempresa=p_IdEmpresa;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: L''empresa ja existeix');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''empresa no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Opci=' || p_opcio ||', IdEmp.=' || p_IdEmpresa || ', Pais='|| p_PaisRegistre || 
            ', Rep.Legal' || p_Representant || ', Adrea=' || p_Adrea || ', Tel.=' ||  p_Telefon); 
    Util.InsertarLog('AM_Empresa:' || v_log_opcio, v_log, rsp);
  
  END AM_Empresa;

---------------------------------------------------------------------  
-- Procediment de baixas d'empreses
-------------------------------------------------------------------    
  PROCEDURE Baixa_Empresa(
    p_IdEmpresa IN empresa.idempresa%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM EMPRESA WHERE idempresa=p_IdEmpresa;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''empresa no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: L''empresa te desenvolupadors. No es pot donar de baixa';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;
   
    -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdEmp.=' || p_IdEmpresa);
    Util.InsertarLog('Baixa_Empresa', v_log, rsp);
    
  END Baixa_Empresa;

--------------------------------------------------------------------------------------  
-- Procediment d'alta i modificaci dels desenvolupadors d'aplicacions
--------------------------------------------------------------------------------------
  PROCEDURE AM_Desenvolupador(
    p_opcio IN CHAR,
    p_IdDesenvolupador IN desenvolupador.iddesenvolupador%TYPE,
    p_Empresa IN desenvolupador.empresa%TYPE,
    p_CodiDesenvolupador IN desenvolupador.codidesenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO DESENVOLUPADOR (iddesenvolupador, empresa, codidesenvolupador) 
            VALUES (p_IdDesenvolupador, p_Empresa, p_CodiDesenvolupador);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE DESENVOLUPADOR SET empresa=p_Empresa, codidesenvolupador=p_CodiDesenvolupador
                WHERE iddesenvolupador=p_IdDesenvolupador;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          IF v_log_opcio='Alta' THEN
            rsp:= ('ERROR: El desenvolupador i/o el codi del desenvolupador ja existeixen');
          ELSE
            rsp:= ('ERROR: El codi del desenvolupador ja existeix');
          END IF;
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: El desenvolupador no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments
      -------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdDesenv.=' || p_IdDesenvolupador || 
              ', Empresa=' ||  p_Empresa || ', Codi=' || p_CodiDesenvolupador); 
      Util.InsertarLog('AM_Desenvolupador:' || v_log_opcio, v_log, rsp);

  END AM_Desenvolupador;
    
--------------------------------------------------------------------------------  
-- Procediment de baixa de desenvolupadors d'aplicacions
--------------------------------------------------------------------------------
  PROCEDURE Baixa_Desenvolupador(
    p_IdDesenvolupador IN desenvolupador.iddesenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      DELETE FROM DESENVOLUPADOR WHERE iddesenvolupador=p_IdDesenvolupador;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:='ERROR: El desenvolupador no existeix.';
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: El desenvolupador te aplicacions desenvolupades. No es pot donar de baixa';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;
  
      -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdDesenv.=' || p_IdDesenvolupador);
    Util.InsertarLog('Baixa_Desenvolupador', v_log, rsp);  
  
  END Baixa_Desenvolupador;  
  
END MantenimentDesenvolupadors;
/