-------------------------------------------------------------------------
--  PAQUET DE MANTENIMENT DELS USUARIS I ENTITATS ASSOCIADES
-- Els procediments d'alta i modificaci s'engloban en un de sol,
-- seleccionant la opci corresponent amb el primer parmetre d'entrada.
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE MantenimentUsuaris IS

  PROCEDURE AM_Dispositiu(
    p_Opcio IN CHAR,
    p_Imei IN dispositiu.imei%TYPE,
    p_IdPropietari IN dispositiu.idpropietari%TYPE,
    p_SoDispositiu IN dispositiu.sodispositiu%TYPE,
    p_ModelDisp IN dispositiu.modeldisp%TYPE,
    p_Resolucio IN dispositiu.resolucio%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Dispositiu(
    p_Imei IN dispositiu.imei%TYPE,
    rsp OUT NOCOPY VARCHAR2);
   
  PROCEDURE AM_Usuari(
    p_opcio IN CHAR,
    p_IdUsuari IN usuari.idusuari%TYPE,
    p_Operador IN usuari.operadorusuari%TYPE,
    p_NumeroMobil IN usuari.numeromobil%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Usuari(
    p_IdUsuari IN usuari.idusuari%TYPE,
    rsp OUT NOCOPY VARCHAR2);
   
END MantenimentUsuaris;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY MantenimentUsuaris IS

  v_log logsprocediments.parametresentrada%type;
  v_log_opcio VARCHAR2(12);
  resHoritzontal VARCHAR2(10);
  resVertical VARCHAR2(10);
  v_propietari dispositiu.idpropietari%TYPE;
  v_contador NUMBER;
  OPCIO_INCORRECTE EXCEPTION;
  NO_TROBAT EXCEPTION;
  FORMAT_RESOLUCIO_INCORRECTE EXCEPTION;
  MOD_INCORRECTA_PROPIETARI EXCEPTION;
  
  
--------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions de dispositius mbils
--------------------------------------------------------------------------------
  PROCEDURE AM_Dispositiu(
    p_Opcio IN CHAR,
    p_Imei IN dispositiu.imei%TYPE,
    p_IdPropietari IN dispositiu.idpropietari%TYPE,
    p_SoDispositiu IN dispositiu.sodispositiu%TYPE,
    p_ModelDisp IN dispositiu.modeldisp%TYPE,
    p_Resolucio IN dispositiu.resolucio%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      resHoritzontal:=substr(p_Resolucio,1,instr(p_Resolucio,'x')-1);
      resVertical:= substr(p_Resolucio,instr(p_Resolucio,'x')+1,length(p_Resolucio)-instr(p_Resolucio,'x'));
       IF resHoritzontal IS NULL OR resVertical IS NULL THEN  
          RAISE FORMAT_RESOLUCIO_INCORRECTE;
      ELSE
          resHoritzontal:=translate(resHoritzontal,'T0123456789','T');
          resVertical:=translate(resVertical,'T0123456789','T');
          IF resHoritzontal IS NOT NULL OR resVertical IS NOT NULL THEN 
                RAISE FORMAT_RESOLUCIO_INCORRECTE;
          END IF;
      END IF;

      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO DISPOSITIU (imei, idpropietari, sodispositiu, modeldisp, resolucio) 
            VALUES (p_Imei, p_IdPropietari, p_SoDispositiu, p_ModelDisp, p_Resolucio);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            
            -- Control per no modificar el propietari del dispositiu si t alguna descrrega feta amb aquest dispositiu.
            SELECT idpropietari INTO v_propietari FROM DISPOSITIU WHERE imei=p_Imei;
            IF p_IdPropietari<> v_propietari THEN
              SELECT COUNT (*) INTO v_contador FROM DESCARREGA WHERE imei=p_Imei;
              IF v_contador > 0 THEN
                RAISE MOD_INCORRECTA_PROPIETARI;
              END IF;
            END IF;
            
            UPDATE DISPOSITIU SET idpropietari=p_IdPropietari, sodispositiu=p_SoDispositiu, 
                modeldisp=p_ModelDisp, resolucio=p_Resolucio
                WHERE imei=p_Imei;
            
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: El dispositiu ja existeix');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_DATA_FOUND THEN
          rsp:= ('ERROR: El dispositiu no existeix.');
          ROLLBACK;
      WHEN MOD_INCORRECTA_PROPIETARI THEN
          rsp:= ('ERROR: Aquest dispositiu t descrregues. No es pot modificar el propietari.');
          ROLLBACK;
      WHEN FORMAT_RESOLUCIO_INCORRECTE THEN
          rsp:= ('ERROR: El format de resoluci no es correcte. Ha de ser ''DIGITSxDIGITS''- Exemple: 480x800');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Opci=' || p_opcio ||', Imei=' || p_Imei || ', Propietari='|| p_IdPropietari || 
            ', SoDisp.' || p_SoDispositiu || ', Model=' || p_ModelDisp || ', Resoluci=' ||  p_Resolucio); 
    Util.InsertarLog('AM_Dispositiu:' || v_log_opcio, v_log, rsp);
  
  END AM_Dispositiu;

---------------------------------------------------------------------  
-- Procediment de baixas de dispositius mbils
-------------------------------------------------------------------    
  PROCEDURE Baixa_Dispositiu(
    p_Imei IN dispositiu.imei%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM DISPOSITIU WHERE imei=p_Imei;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: El dispositiu no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: S''han fet descrregues amb aquest dispositiu. No es pot donar de baixa.';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
    END;
   
    -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('Imei=' || p_Imei);
    Util.InsertarLog('Baixa_Dispositiu', v_log, rsp);
    
  END Baixa_Dispositiu;

--------------------------------------------------------------------------------------  
-- Procediment d'alta i modificaci d'usuaris 
--------------------------------------------------------------------------------------
  PROCEDURE AM_Usuari(
    p_opcio IN CHAR,
    p_IdUsuari IN usuari.idusuari%TYPE,
    p_Operador IN usuari.operadorusuari%TYPE,
    p_NumeroMobil IN usuari.numeromobil%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO USUARI (idusuari, operadorusuari, numeromobil) 
            VALUES (p_IdUsuari, p_Operador, p_NumeroMobil);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE USUARI SET operadorusuari=p_Operador, numeromobil=p_NumeroMobil
                WHERE idusuari=p_IdUsuari;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          IF v_log_opcio='Alta' THEN
            rsp:= ('ERROR: L''usuari i/o el nmero de mbil ja existeixen');
          ELSE
            rsp:= ('ERROR: El nmero de mbil ja existeix');
          END IF;
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''usuari no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments
      -------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdUsuari=' || p_IdUsuari || 
              ', Operador=' || p_Operador || ', Mbil=' || p_NumeroMobil); 
      Util.InsertarLog('AM_Usuari:' || v_log_opcio, v_log, rsp);

  END AM_Usuari;
    
--------------------------------------------------------------------------------  
-- Procediment de baixa d'usuaris
--------------------------------------------------------------------------------
  PROCEDURE Baixa_Usuari(
    p_IdUsuari IN usuari.idusuari%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      DELETE FROM USUARI WHERE idusuari=p_IdUsuari;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:='ERROR: L''usuari no existeix.';
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: L''usuari t dispositius mbils. No es pot donar de baixa';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;
  
      -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdUsuari=' || p_IdUsuari);
    Util.InsertarLog('Baixa_Usuari', v_log, rsp);  
  
  END Baixa_Usuari;  
  
END MantenimentUsuaris;
/