-------------------------------------------------------------------------
--                  PAQUET D'UTILITATS
-- Inclou les insercions de logs i el manteniment de la taula PERSONES
-- Els procediments d'alta i modificaci s'engloban en un de sol,
-- seleccionant la opci corresponent amb el primer parmetre d'entrada.
-- Tamb conte una declaracio de variables globals que seran utilitzades 
-- per els disparadors del modul estadstic.
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE Util IS

    --  Declaraci de variables globals utilitzades en el mdul estadstic.

    v_anydescarregaNova NUMBER;
    v_anydescarregaVella NUMBER;
    v_imeiNou descarrega.imei%TYPE;
    v_imeiVell descarrega.imei%TYPE;
    v_paisNou descarrega.pais%TYPE;
    v_paisVell descarrega.pais%TYPE;
    v_aplicacioNova descarrega.aplicacio%TYPE;
    v_aplicacioVella descarrega.aplicacio%TYPE;
    v_preuNou descarrega.preu%TYPE;
    v_preuVell descarrega.preu%TYPE;
   

  PROCEDURE InsertarLog (
    p_procediment IN VARCHAR2,
    p_parEntrada IN logsprocediments.parametresentrada%type,
    p_parSortida IN OUT NOCOPY VARCHAR2) ;
      
  PROCEDURE AM_Persona (
    p_Opcio IN CHAR,
    p_Identificador IN persona.idpersona%TYPE,
    p_nom IN persona.nom%TYPE,
    p_cognoms IN persona.cognoms%TYPE,
    p_adrea IN persona.adrea%TYPE,
    p_Poblacio IN persona.poblacio%TYPE, 
    p_Pais IN persona.pais%TYPE,
    p_telefon IN persona.telefon%TYPE,
    p_email IN persona.email%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Persona (
    p_Identificador IN persona.idpersona%TYPE,
    rsp OUT NOCOPY VARCHAR2);
      
END Util;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY Util IS
               
  v_log logsprocediments.parametresentrada%type;
  v_log_opcio VARCHAR2(12);
  OPCIO_INCORRECTE EXCEPTION;
  NO_TROBAT EXCEPTION;

----------------------------------------------------------------------------
--   Insercions en la taula de logs
----------------------------------------------------------------------------
  PROCEDURE InsertarLog (
    p_procediment IN VARCHAR2,
    p_parEntrada IN logsprocediments.parametresentrada%type,
    p_parSortida IN OUT NOCOPY VARCHAR2) IS
    
  v_resposta_log VARCHAR2(255);
  BEGIN
    BEGIN
      INSERT INTO logsprocediments( nomprocediment, parametresentrada, parametressortida) 
          VALUES (p_procediment, p_parEntrada, p_parSortida);   
      v_resposta_log:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN OTHERS THEN 
        v_resposta_log:='ERROR:'||SQLERRM;
            
    END;

    -- En cas de que hagi un error en la inserci a la taula de logs, es retorna
    -- un Warning amb el tipus d'error concatenat al parmetre de resposta del procediment
    IF v_resposta_log <> 'OK' THEN
      p_parSortida:=p_parSortida || '  - Warning: Taula logs no actualitzada: ' || v_resposta_log;
    END IF;
    
  END InsertarLog;
  
--------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions de la taula PERSONES
--------------------------------------------------------------------------------  
  PROCEDURE AM_Persona (
    p_Opcio IN CHAR,
    p_Identificador IN persona.idpersona%TYPE,
    p_Nom IN persona.nom%TYPE,
    p_Cognoms IN persona.cognoms%TYPE,
    p_Adrea IN persona.adrea%TYPE,
    p_Poblacio IN persona.poblacio%TYPE,
    p_Pais IN persona.pais%TYPE,
    p_Telefon IN persona.telefon%TYPE,
    p_Email IN persona.email%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO PERSONA (idpersona, nom, cognoms, adrea, telefon, poblacio, pais, email) 
            VALUES (p_Identificador, p_Nom, p_Cognoms, p_Adrea, p_Telefon, p_Poblacio, p_Pais, p_Email);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE PERSONA SET nom=p_Nom, cognoms=p_Cognoms, adrea=p_Adrea, 
                telefon=p_Telefon, poblacio=p_Poblacio, pais=p_Pais, email=p_Email
                WHERE idpersona=p_Identificador;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: La persona ja existeix');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: La persona no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Opci=' || p_opcio ||', IdPersona.=' || p_Identificador || ', Nom='|| p_Nom || ', Cognoms=' || p_Cognoms || 
            ', Adrea=' || p_Adrea || ', Poblaci=' || p_Poblacio || ', Pais=' || p_Pais ||', Tel.=' || p_Telefon || ', Email=' ||  p_Email); 
    InsertarLog('AM_Persona:' || v_log_opcio, v_log, rsp);
  
  END AM_Persona;
  
---------------------------------------------------------------------  
-- Procediment de baixa de Persones
-------------------------------------------------------------------    
  PROCEDURE Baixa_Persona (
    p_Identificador IN persona.idpersona%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM PERSONA WHERE idpersona=p_Identificador;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: La persona no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: La persona es un usuari, un desenvolupador o un representant legal. No es pot donar de baixa.';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;
   
    -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdPersona=' || p_Identificador);
    InsertarLog('Baixa_Persona', v_log, rsp);
    
  END Baixa_Persona;

END Util;
/