--------------------------------------------------------------------------------
--                MDUL ESTADISTIC.  Consulta 3
--  S'encarrega de donar resposta a la consulta del nmero mig d'aplicacions
--  descarregades per usuari en un any donat.
--  Consta de dos Triggers: Un a nivell de fila, que s'encarrega d'emmagatzemar
--  les dades :new i :old que despres utilitzarem per la consulta. Es important
--  fer-ho aix per que en cas contrari, es produirien errors de 'Taules Mutants'
--  al intentar llegir o modificar una d'aquestes taules dins del cos del 
--  disparador (en el nostre cas la taula DESCARREGA).
--  L'altra disparador es a nivell de ordre (statement) i utilitza les variables
--  assignades anteriorment, les quals estan definides com a variables globals
--  dins del paquet 'UTIL'.
--------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER Estad_3_mitja_apps_row
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

BEGIN
    
    util.v_imeiNou:=:new.imei;
    util.v_imeiVell:=:old.imei;
    util.v_anydescarregaNova:=EXTRACT(YEAR FROM :new.datadescarrega);
    util.v_anydescarregaVella:=EXTRACT(YEAR FROM :old.datadescarrega);

END Estad_3_mitja_apps_row; 
/


CREATE OR REPLACE TRIGGER Estad_3_mitja_apps_statement
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA


DECLARE
  v_usuaris NUMBER;
  v_aplicacions NUMBER;
  
  PROCEDURE seleccio_anyDescarregaNou IS
  BEGIN
      SELECT COUNT(DISTINCT idUsuari), COUNT(*)  INTO v_usuaris, v_aplicacions
      FROM (descarrega NATURAL JOIN dispositiu) JOIN usuari ON idPropietari=idUsuari
      WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaNova;
   
      UPDATE ESTADISTICA SET valor=v_aplicacions/v_usuaris
      WHERE codiestadistica=3 AND anyest=util.v_anydescarregaNova;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, anyest, valor) 
        VALUES (3, util.v_anydescarregaNova, v_aplicacions/v_usuaris);
      END IF;
  END seleccio_anyDescarregaNou;
  
  PROCEDURE seleccio_anyDescarregaVell IS
  BEGIN
      SELECT COUNT(DISTINCT idUsuari), COUNT(*)  INTO v_usuaris, v_aplicacions
      FROM (descarrega NATURAL JOIN dispositiu) JOIN usuari ON idPropietari=idUsuari
      WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaVella;
   
        -- Es controla si quedan usuaris que hagin baixat aplicacions aquest any. Si no ni ha, es borra el registre
        -- corresponent de la taula Estadstica.
      IF v_usuaris > 0 THEN
        UPDATE ESTADISTICA SET valor=v_aplicacions/v_usuaris
        WHERE codiestadistica=3 AND anyest=util.v_anydescarregaVella;
      ELSE
        DELETE FROM ESTADISTICA
        WHERE codiestadistica=3 AND anyest=util.v_anydescarregaVella;
      END IF;
  END seleccio_anyDescarregaVell;
  
BEGIN
  
  IF INSERTING THEN
      seleccio_anyDescarregaNou;

  ELSIF UPDATING THEN
  -- Es controla si s'han canviat els valors de 'Imei' o l'any de descrrega, que son els que poden
  -- afectar a la mitja d'aplicacions descarregades per usuari en un any donat.
      IF util.v_imeiNou <> util.v_imeiVell OR util.v_anydescarregaNova <> util.v_anydescarregaVella THEN 
        seleccio_anyDescarregaNou;
        seleccio_anyDescarregaVell;
      END IF;

  ELSIF DELETING THEN
      seleccio_anyDescarregaVell;
 
  END IF;

END Estad_3_mitja_apps_statement; 
/

