--------------------------------------------------------------------------------
--                  MDUL ESTADISTIC.  Consulta 4
--  S'encarrega de donar resposta a la consulta del desenvolupador que ha tingut
--  el mxim numero de descarregues de les seves aplicacions en un any donat.
--  Consta de dos Triggers: Un a nivell de fila, que s'encarrega d'emmagatzemar
--  les dades :new i :old que despres utilitzarem per la consulta. Es important
--  fer-ho aix per que en cas contrari, es produirien errors de 'Taules Mutants'
--  al intentar llegir o modificar una d'aquestes taules dins del cos del 
--  disparador (en el nostre cas la taula DESCARREGA).
--  L'altra disparador es a nivell de ordre (statement) i utilitza les variables
--  assignades anteriorment, les quals estan definides com a variables globals
--  dins del paquet 'UTIL'.
--------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER Estad_4_desc_desenv_row
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

BEGIN
    
    util.v_aplicacioNova:=:new.aplicacio;
    util.v_aplicacioVella:=:old.aplicacio;
    util.v_anydescarregaNova:=EXTRACT(YEAR FROM :new.datadescarrega);
    util.v_anydescarregaVella:=EXTRACT(YEAR FROM :old.datadescarrega);

END Estad_4_desc_desenv_row; 
/


CREATE OR REPLACE TRIGGER Estad_4_desc_desenv_statement
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA


DECLARE
  v_desenvolupador desenvolupador.iddesenvolupador%TYPE;
  v_aplicacions NUMBER;
  
  PROCEDURE seleccio_anyDescarregaNou IS
  BEGIN
      SELECT desenvolupador, apps_descarregades INTO v_desenvolupador, v_aplicacions FROM
        (SELECT  apl.desenvolupador, count(*) as apps_descarregades
         FROM (descarrega de JOIN aplicacio ap ON de.aplicacio=ap.idaplicacio) 
             JOIN apldesenvolupada apl ON apl.aplicacio=ap.idaplicacio
         WHERE EXTRACT(YEAR FROM de.datadescarrega)=util.v_anydescarregaNova
         GROUP BY apl.desenvolupador
         ORDER BY apps_descarregades DESC, apl.desenvolupador ASC)
      WHERE ROWNUM=1;
   
      UPDATE ESTADISTICA SET valor=v_aplicacions, desenvolupador=v_desenvolupador
      WHERE codiestadistica=4 AND anyest=util.v_anydescarregaNova;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, anyest, valor, desenvolupador) 
        VALUES (4, util.v_anydescarregaNova, v_aplicacions, v_desenvolupador);
      END IF;
  END seleccio_anyDescarregaNou;
  
  PROCEDURE seleccio_anyDescarregaVell IS
  BEGIN
      SELECT desenvolupador, apps_descarregades INTO v_desenvolupador, v_aplicacions FROM
        (SELECT  apl.desenvolupador, count(*) as apps_descarregades
         FROM (descarrega de JOIN aplicacio ap ON de.aplicacio=ap.idaplicacio) 
             JOIN apldesenvolupada apl ON apl.aplicacio=ap.idaplicacio
         WHERE EXTRACT(YEAR FROM de.datadescarrega)=util.v_anydescarregaVella
         GROUP BY apl.desenvolupador
         ORDER BY apps_descarregades DESC, apl.desenvolupador ASC)
      WHERE ROWNUM=1;
   
      UPDATE ESTADISTICA SET valor=v_aplicacions, desenvolupador=v_desenvolupador
      WHERE codiestadistica=4 AND anyest=util.v_anydescarregaVella;
              
  EXCEPTION
    -- Es controla si quedan aplicacions que s'hagin descarregat aquest any. Si no ni ha, es borra el registre
    -- corresponent de la taula Estadstica.
    WHEN NO_DATA_FOUND THEN
      DELETE FROM ESTADISTICA
      WHERE codiestadistica=4 AND anyest=util.v_anydescarregaVella;

  END seleccio_anyDescarregaVell;
  
BEGIN
  
  IF INSERTING THEN
      seleccio_anyDescarregaNou;

  ELSIF UPDATING THEN
  -- Es controla si s'han canviat els valors de 'Aplicacio' o l'any de descrrega, que son els que poden
  -- afectar al desenvolupador que te el mxim numero de descrregues en un any donat.
      IF util.v_aplicacioNova <> util.v_aplicacioVella OR util.v_anydescarregaNova <> util.v_anydescarregaVella THEN 
        seleccio_anyDescarregaNou;
        seleccio_anyDescarregaVell;
      END IF;

  ELSIF DELETING THEN
      seleccio_anyDescarregaVell;
 
  END IF;

END Estad_4_desc_desenv_statement; 
/

