--------------------------------------------------------------------------------
--                  MDUL ESTADISTIC.  Consulta 5
--  S'encarrega de donar resposta a la consulta de l'aplicacio que mes diners
--  ha recaudat en descarregues, aixi com el seu desenvolupador, en un any donat.
--  Consta de dos Triggers: Un a nivell de fila, que s'encarrega d'emmagatzemar
--  les dades :new i :old que despres utilitzarem per la consulta. Es important
--  fer-ho aix per que en cas contrari, es produirien errors de 'Taules Mutants'
--  al intentar llegir o modificar una d'aquestes taules dins del cos del 
--  disparador (en el nostre cas la taula DESCARREGA).
--  L'altra disparador es a nivell de ordre (statement) i utilitza les variables
--  assignades anteriorment, les quals estan definides com a variables globals
--  dins del paquet 'UTIL'.
--------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER Estad_5_app_maxRecaudacio_row
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

BEGIN
    
    util.v_aplicacioNova:=:new.aplicacio;
    util.v_aplicacioVella:=:old.aplicacio;
    util.v_anydescarregaNova:=EXTRACT(YEAR FROM :new.datadescarrega);
    util.v_anydescarregaVella:=EXTRACT(YEAR FROM :old.datadescarrega);

END Estad_5_app_maxRecaudacio_row; 
/


CREATE OR REPLACE TRIGGER Estad_5_app_maxRecaudacio_stat
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA

DECLARE
  v_desenvolupador estadistica.desenvolupador%TYPE;
  v_aplicacio aplicacio.idaplicacio%TYPE;
  v_total_desenvolupadors NUMBER;
  
  PROCEDURE seleccio_anyDescarregaNou IS
  BEGIN
      SELECT aplicacio INTO v_aplicacio FROM DESCARREGA
         WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaNova
         GROUP BY aplicacio
         HAVING sum(preu)=
            (SELECT max(sum(preu)) 
             FROM descarrega 
             WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaNova
             GROUP BY aplicacio );
             
      SELECT desenvolupador INTO v_desenvolupador FROM
         (SELECT desenvolupador FROM APLDESENVOLUPADA
          WHERE aplicacio=v_aplicacio)
      WHERE ROWNUM=1;
      
      SELECT count(*) INTO v_total_desenvolupadors FROM APLDESENVOLUPADA
          WHERE aplicacio=v_aplicacio;
      
      v_desenvolupador:=v_desenvolupador || ' , i ' || TO_CHAR(v_total_desenvolupadors-1) || ' mes.';
      UPDATE ESTADISTICA SET  desenvolupador=v_desenvolupador, aplicacio=v_aplicacio
      WHERE codiestadistica=5 AND anyest=util.v_anydescarregaNova;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, anyest, desenvolupador, aplicacio) 
        VALUES (5, util.v_anydescarregaNova, v_desenvolupador, v_aplicacio);
      END IF;
  END seleccio_anyDescarregaNou;
  
  PROCEDURE seleccio_anyDescarregaVell IS
  BEGIN
     SELECT aplicacio INTO v_aplicacio FROM DESCARREGA
         WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaVella
         GROUP BY aplicacio
         HAVING sum(preu)=
            (SELECT max(sum(preu)) 
             FROM descarrega 
             WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaVella
             GROUP BY aplicacio );
             
      SELECT desenvolupador INTO v_desenvolupador FROM
         (SELECT desenvolupador FROM APLDESENVOLUPADA
          WHERE aplicacio=v_aplicacio)
      WHERE ROWNUM=1;
      
      SELECT count(*) INTO v_total_desenvolupadors FROM APLDESENVOLUPADA
          WHERE aplicacio=v_aplicacio;
   
      v_desenvolupador:=v_desenvolupador || ' , i ' || TO_CHAR(v_total_desenvolupadors-1) || ' mes.';
      UPDATE ESTADISTICA SET  desenvolupador=v_desenvolupador, aplicacio=v_aplicacio
      WHERE codiestadistica=5 AND anyest=util.v_anydescarregaVella;
              
  EXCEPTION
    -- Es controla si quedan aplicacions que s'hagin descarregat aquest any. Si no ni ha, es borra el registre
    -- corresponent de la taula Estadstica.
    WHEN NO_DATA_FOUND THEN
      DELETE FROM ESTADISTICA
      WHERE codiestadistica=5 AND anyest=util.v_anydescarregaVella;

  END seleccio_anyDescarregaVell;
  
BEGIN
  
  IF INSERTING THEN
      seleccio_anyDescarregaNou;

  ELSIF UPDATING THEN
  -- Es controla si s'han canviat els valors de 'Aplicacio' o l'any de descrrega, que son els que poden
  -- afectar a l'aplicaci que mes diners ha recaudat en descarregues en un any donat.
      IF util.v_aplicacioNova <> util.v_aplicacioVella OR util.v_anydescarregaNova <> util.v_anydescarregaVella THEN 
        seleccio_anyDescarregaNou;
        seleccio_anyDescarregaVell;
      END IF;

  ELSIF DELETING THEN
      seleccio_anyDescarregaVell;
 
  END IF;

END Estad_5_app_maxRecaudacio_stat; 
/

