--------------------------------------------------------------------------------
--                  MDUL ESTADISTIC.  Consulta 8
--  S'encarrega de donar resposta a la consulta del nmero d'aplicacions
--  diferents descarregades al menys una vegada en un pais i un any determinat.
--  Consta de dos Triggers: Un a nivell de fila, que s'encarrega d'emmagatzemar
--  les dades :new i :old que despres utilitzarem per la consulta. Es important
--  fer-ho aix per que en cas contrari, es produirien errors de 'Taules Mutants'
--  al intentar llegir o modificar una d'aquestes taules dins del cos del 
--  disparador (en el nostre cas la taula DESCARREGA).
--  L'altra disparador es a nivell de ordre (statement) i utilitza les variables
--  assignades anteriorment, les quals estan definides com a variables globals
--  dins del paquet 'UTIL'.
--------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER Estad_8_Apps_Diferents_row
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

BEGIN
    
    util.v_paisNou:=:new.pais;
    util.v_paisVell:=:old.pais;
    util.v_anydescarregaNova:=EXTRACT(YEAR FROM :new.datadescarrega);
    util.v_anydescarregaVella:=EXTRACT(YEAR FROM :old.datadescarrega);
    util.v_aplicacioNova:=:new.aplicacio;
    util.v_aplicacioVella:=:old.aplicacio;

END Estad_8_Apps_Diferents_row; 
/


CREATE OR REPLACE TRIGGER Estad_8_Apps_Diferents_stat
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA

DECLARE

  v_total_aplicacions NUMBER;
  
  PROCEDURE seleccio_anyDescarregaNou IS
  BEGIN
      SELECT count(*) INTO v_total_aplicacions FROM
        (SELECT aplicacio FROM descarrega
         WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaNova AND pais = util.v_paisNou
         GROUP BY aplicacio);
  
      UPDATE ESTADISTICA SET  valor=v_total_aplicacions
      WHERE codiestadistica=8 AND anyest=util.v_anydescarregaNova AND pais=util.v_paisNou;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, anyest, pais, valor) 
        VALUES (8, util.v_anydescarregaNova, util.v_paisNou, v_total_aplicacions);
      END IF;
      
  END seleccio_anyDescarregaNou;
  
  PROCEDURE seleccio_anyDescarregaVell IS
  BEGIN
      SELECT count(*) INTO v_total_aplicacions FROM
        (SELECT aplicacio FROM descarrega
         WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaVella AND pais = util.v_paisVell
         GROUP BY aplicacio);

    -- Es controla si quedan aplicacions descarregades del pais i any donats. 
    -- Si no ni ha, es borra el registre corresponent de la taula Estadstica.
      IF v_total_aplicacions=0 THEN
        DELETE FROM ESTADISTICA
        WHERE codiestadistica=8 AND anyest=util.v_anydescarregaVella AND pais=util.v_paisVell;
      ELSE
        UPDATE ESTADISTICA SET valor=v_total_aplicacions
        WHERE codiestadistica=8 AND anyest=util.v_anydescarregaVella AND pais=util.v_paisVell;
      END IF;
      
  END seleccio_anyDescarregaVell;
  
BEGIN
  
  IF INSERTING THEN
      seleccio_anyDescarregaNou;

  ELSIF UPDATING THEN
  -- Es controla si s'han canviat els valors del Pais, l'any o l'aplicaci, que son els que poden
  -- afectar al nmero d'aplicacions descarregades diferents al menys una vegada.
      IF util.v_paisNou <> util.v_paisVell OR util.v_anydescarregaNova <> util.v_anydescarregaVella 
            OR util.v_aplicacioNova <> util.v_aplicacioVella THEN 
        seleccio_anyDescarregaNou;
        seleccio_anyDescarregaVell;
      END IF;

  ELSIF DELETING THEN
      seleccio_anyDescarregaVell;
 
  END IF;

END Estad_8_Apps_Diferents_stat; 
/

