 -------------------------------------------------------------------------------
--  Script encarregat de la creaci de totes les taules, ndexs i disparadors 
--                       inicials del sistema
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- 1. Creacio de les taules d'entitats auxiliars                                
-------------------------------------------------------------------------------
CREATE TABLE OPERADOR (
  nomOperador VARCHAR2(30) CONSTRAINT PK_OPERADOR PRIMARY KEY
);
-------------------------------------------------------------------------------
CREATE TABLE PAIS (
  idPais CHAR(2) CONSTRAINT PK_PAIS PRIMARY KEY,
  nom VARCHAR2(50) CONSTRAINT NN_PAIS_nom NOT NULL
          CONSTRAINT UK_PAIS_nom UNIQUE
);
-------------------------------------------------------------------------------
CREATE TABLE IDIOMA (
  codiIdioma CHAR(2) CONSTRAINT PK_IDIOMA PRIMARY KEY,
  nomIdioma VARCHAR2(20) CONSTRAINT NN_IDIOMA_nomIdioma NOT NULL
          CONSTRAINT UK_IDIOMA_nomIdioma UNIQUE
);
-------------------------------------------------------------------------------
CREATE TABLE SISTOPERATIU (
  nomSO VARCHAR2(30) CONSTRAINT PK_SISTOPERATIU PRIMARY KEY
);
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- 2. Creacio de les taules d'entitats principals    
-------------------------------------------------------------------------------
CREATE TABLE APLICACIO (
  idAplicacio VARCHAR2(30) CONSTRAINT PK_APLICACIO PRIMARY KEY,
  versio VARCHAR2(20) CONSTRAINT NN_APLICACIO_versio NOT NULL,
  dataPujada DATE CONSTRAINT NN_APLICACIO_dataPujada NOT NULL,
  url VARCHAR2(255),
  resolucioMin VARCHAR2(10) CONSTRAINT NN_APLICACIO_resolucioMin NOT NULL,
  activa NUMBER(1) CONSTRAINT NN_APLICACIO_activa NOT NULL,
  CONSTRAINT CHK_APLICACIO_activa CHECK (activa=0 OR activa=1)
);
-------------------------------------------------------------------------------
CREATE TABLE PERSONA (
  idPersona VARCHAR2(15) CONSTRAINT PK_PERSONA PRIMARY KEY,
  nom VARCHAR2(20) CONSTRAINT NN_PERSONA_nom NOT NULL,
  cognoms VARCHAR2(40) CONSTRAINT NN_PERSONA_cognoms NOT NULL,
  adrea VARCHAR2(50),
  poblacio VARCHAR2(40),
  pais CHAR(2) 
          CONSTRAINT FK_PERSONA_pais REFERENCES PAIS(idPais)
          CONSTRAINT NN_PERSONA_pais NOT NULL,
  telefon VARCHAR2(15),
  email VARCHAR2(40) CONSTRAINT NN_PERSONA_email NOT NULL
);

CREATE INDEX XPERSONA_pais ON PERSONA(pais);
-------------------------------------------------------------------------------
CREATE TABLE EMPRESA (
  idEmpresa VARCHAR2(30) CONSTRAINT PK_EMPRESA PRIMARY KEY,
  paisRegistre CHAR(2) 
          CONSTRAINT FK_EMPRESA_pais REFERENCES PAIS(idPais)
          CONSTRAINT NN_EMPRESA_paisRegistre NOT NULL,
  representant VARCHAR2(15) 
          CONSTRAINT FK_EMPRESA_representant REFERENCES PERSONA(idPersona)
          CONSTRAINT NN_EMPRESA_representant NOT NULL,
  adrea VARCHAR2(50),
  poblacio VARCHAR2(40),
  telefon VARCHAR2(15)
);
-------------------------------------------------------------------------------
CREATE TABLE DESENVOLUPADOR (
  idDesenvolupador VARCHAR2(15) 
          CONSTRAINT PK_DESENVOLUPADOR PRIMARY KEY
          CONSTRAINT FK_DESENVOLUPADOR_idDes REFERENCES PERSONA(idPersona),
  empresa VARCHAR2(30) 
          CONSTRAINT NN_DESENVOLUPADOR_empresa NOT NULL
          CONSTRAINT FK_DESENVOLUPADOR_empresa REFERENCES EMPRESA(idEmpresa),
  codiDesenvolupador CHAR(10)
          CONSTRAINT NN_DESENVOLUPADOR_codiDes NOT NULL
          CONSTRAINT UK_DESENVOLUPADOR_codiDes UNIQUE
);

CREATE INDEX XDESENVOLUPADOR_empresa ON DESENVOLUPADOR(empresa);
-------------------------------------------------------------------------------
CREATE TABLE USUARI (
  idUsuari VARCHAR2(15) 
          CONSTRAINT PK_USUARI PRIMARY KEY
          CONSTRAINT FK_USUARI_idUsuari REFERENCES PERSONA(idPersona),
  operadorUsuari VARCHAR2(30) 
          CONSTRAINT FK_USUARI_operadorUsuari REFERENCES OPERADOR(nomOperador)
          CONSTRAINT NN_USUARI_operadorUsuari NOT NULL,
  numeroMobil VARCHAR2(15) 
          CONSTRAINT NN_USUARI_numeroMobil NOT NULL
          CONSTRAINT UK_USUARI_numeroMobil UNIQUE
 );

CREATE INDEX XUSUARI_operadorUsuari ON USUARI(operadorUsuari);
-------------------------------------------------------------------------------
CREATE TABLE DISPOSITIU (
  imei VARCHAR2(15) CONSTRAINT PK_DISPOSITIU PRIMARY KEY,
  idPropietari VARCHAR2(15) 
          CONSTRAINT FK_DISPOSITIU_idPropietari REFERENCES USUARI(idUsuari)
          CONSTRAINT NN_DISPOSITIU_idPropietari NOT NULL,
  soDispositiu VARCHAR2(30) 
          CONSTRAINT FK_DISPOSITIU_soDispositiu REFERENCES SISTOPERATIU(nomSO)
          CONSTRAINT NN_DISPOSITIU_soDispositiu NOT NULL,
  modelDisp VARCHAR2(30) CONSTRAINT NN_DISPOSITIU_modelDisp NOT NULL,
  resolucio VARCHAR2(10) CONSTRAINT NN_DISPOSITIU_resolucio NOT NULL
);

CREATE INDEX XDISPOSITIU_soDispositiu ON DISPOSITIU(soDispositiu);
CREATE INDEX XDISPOSITIU_idPropietari ON DISPOSITIU(idPropietari);
-------------------------------------------------------------------------------
CREATE TABLE DESCARREGA (
  idDescarrega NUMBER(12) CONSTRAINT PK_DESCARREGA PRIMARY KEY,
  aplicacio VARCHAR2(30) 
          CONSTRAINT FK_DESCARREGA_aplicacio REFERENCES APLICACIO(idAplicacio)
          CONSTRAINT NN_DESCARREGA_aplicacio NOT NULL,
  imei VARCHAR(15) 
          CONSTRAINT FK_DESCARREGA_imei REFERENCES DISPOSITIU(imei)
          CONSTRAINT NN_DESCARREGA_imei NOT NULL,
  pais CHAR(2) 
          CONSTRAINT FK_DESCARREGA_pais REFERENCES PAIS(idPais)
          CONSTRAINT NN_DESCARREGA_pais NOT NULL,
  nomOperador VARCHAR2(30) 
          CONSTRAINT FK_DESCARREGA_nomOperador REFERENCES OPERADOR(nomOperador)
          CONSTRAINT NN_DESCARREGA_nomOperador NOT NULL,
  dataDescarrega DATE CONSTRAINT NN_DESCARREGA_dataDesc NOT NULL,
  preu NUMBER(8,2) 
          CONSTRAINT NN_DESCARREGA_preu NOT NULL
          CONSTRAINT CHK_DESCARREGA_preu CHECK (preu >= 0),
  modePagament VARCHAR(20)
);

CREATE INDEX XDESCARREGA_pais ON DESCARREGA(pais);
CREATE INDEX XDESCARREGA_aplicacio ON DESCARREGA(aplicacio);
CREATE INDEX XDESCARREGA_imei ON DESCARREGA(imei);
CREATE INDEX XDESCARREGA_nomOperador ON DESCARREGA(nomOperador);
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- 3. Creacio de les taules d'entitats que provenen de relacions [n..n]    
-------------------------------------------------------------------------------
CREATE TABLE SISTSUPORTAT (
  aplicacio VARCHAR2(30) 
      CONSTRAINT FK_SISTSUPORTAT_aplicacio 
      REFERENCES APLICACIO(idAplicacio) ON DELETE CASCADE,
  sistemaOperatiu VARCHAR2(30)
      CONSTRAINT FK_SISTSUPORTAT_sistemaOp REFERENCES SISTOPERATIU(nomSO),
  midaAplicacio NUMBER(12) CONSTRAINT NN_SISTSUPORTAT_midaApliacio NOT NULL,
  fitxerBinari VARCHAR(255) CONSTRAINT NN_SISTSUPORTAT_fitxerBinari NOT NULL,
  
  CONSTRAINT PK_SISTSUPORTAT PRIMARY KEY(aplicacio, sistemaOperatiu)
);
-------------------------------------------------------------------------------
CREATE TABLE APLDESCRITA (
  aplicacio VARCHAR2(30) 
      CONSTRAINT FK_APLDESCRITA_aplicacio 
      REFERENCES APLICACIO(idAplicacio) ON DELETE CASCADE,
  idioma CHAR(2)
      CONSTRAINT FK_APLDESCRITA_idioma REFERENCES IDIOMA(codiIdioma),
  descripcio VARCHAR2(1024),
 
  CONSTRAINT PK_APLDESCRITA PRIMARY KEY(aplicacio, idioma)
);
-------------------------------------------------------------------------------
CREATE TABLE APLPREU (
  aplicacio VARCHAR2(30)
      CONSTRAINT FK_APLPREU_aplicacio 
      REFERENCES APLICACIO(idAplicacio) ON DELETE CASCADE,
  pais CHAR(2)
      CONSTRAINT FK_APLPREU_pais REFERENCES PAIS(idPais),
  preu NUMBER(8,2) 
      CONSTRAINT NN_APLPREU_preu NOT NULL
      CONSTRAINT CHK_APLPREU_preu CHECK (preu >= 0),

  CONSTRAINT PK_APLPREU PRIMARY KEY(aplicacio, pais)
);
-------------------------------------------------------------------------------
CREATE TABLE APLDESENVOLUPADA (
  aplicacio VARCHAR2(30)
      CONSTRAINT FK_APLDESENVOLUPADA_aplicacio 
      REFERENCES APLICACIO(idAplicacio) ON DELETE CASCADE,
  desenvolupador VARCHAR2(15) 
      CONSTRAINT FK_APLDESENVOLUPADA_desenv 
      REFERENCES DESENVOLUPADOR(idDesenvolupador),

  CONSTRAINT PK_APLDESENVOLUPADA PRIMARY KEY(aplicacio, desenvolupador)
);
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- 4. Creaci de les taules d'entitats estadstiques i logs 
-------------------------------------------------------------------------------
CREATE TABLE LOGSPROCEDIMENTS (
  idLog NUMBER(12) CONSTRAINT PK_LOGSPROCEDIMENTS PRIMARY KEY,
  nomProcediment VARCHAR2(30) CONSTRAINT NN_LOGSPROCEDIMENTS_nom NOT NULL,
  parametresEntrada VARCHAR2(256) CONSTRAINT NN_LOGSPROCEDIMENTS_parEnt NOT NULL,
  parametresSortida VARCHAR2(256) CONSTRAINT NN_LOGSPROCEDIMENTS_parSor NOT NULL
);
-------------------------------------------------------------------------------
CREATE TABLE CODISESTADISTICS (
  idCodi NUMBER(4) CONSTRAINT PK_CODISESTADISTICS PRIMARY KEY,
  descripcio VARCHAR2(255) CONSTRAINT NN_CODISESTADISTICS_descripcio NOT NULL
);
-------------------------------------------------------------------------------
CREATE TABLE ESTADISTICA (
  idEstadistica NUMBER(6) CONSTRAINT PK_ESTADISTICA PRIMARY KEY,
  codiEstadistica NUMBER(4)
      CONSTRAINT NN_ESTADISTICA_codi NOT NULL
      CONSTRAINT FK_ESTADISTICA_codi REFERENCES CODISESTADISTICS(idCodi),
  anyEst CHAR(4),
  pais CHAR(2),
  valor NUMBER(15,3),
  desenvolupador VARCHAR2(30),
  aplicacio VARCHAR2(30)
);
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- 5. Creaci de les sequncies i disparadors de claus primaries
--    autoincrementals 
-------------------------------------------------------------------------------
CREATE SEQUENCE SEQ_DESCARREGA INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER autoincrement_DESCARREGA
    BEFORE INSERT ON DESCARREGA
    FOR EACH ROW
BEGIN
    SELECT SEQ_DESCARREGA.NEXTVAL INTO :NEW.idDescarrega
    FROM DUAL;
END autoincrement_DESCARREGA;
/
-------------------------------------------------------------------------------
CREATE SEQUENCE SEQ_LOGSPROCEDIMENTS INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER autoincrement_LOGSPROCEDIMENTS
    BEFORE INSERT ON LOGSPROCEDIMENTS
    FOR EACH ROW
BEGIN
    SELECT SEQ_LOGSPROCEDIMENTS.NEXTVAL INTO :NEW.idLog
    FROM DUAL;
END autoincrement_LOGSPROCEDIMENTS;
/
-------------------------------------------------------------------------------
CREATE SEQUENCE SEQ_ESTADISTICA INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER autoincrement_ESTADISTICA
    BEFORE INSERT ON ESTADISTICA
    FOR EACH ROW
BEGIN
    SELECT SEQ_ESTADISTICA.NEXTVAL INTO :NEW.idEstadistica
    FROM DUAL;
END autoincrement_ESTADISTICA;
/
-------------------------------------------------------------------------------
CREATE SEQUENCE SEQ_CODISESTADISTICS INCREMENT BY 1 START WITH 1;

CREATE OR REPLACE TRIGGER autoincrement_CODISESTADISTICS
    BEFORE INSERT ON CODISESTADISTICS
    FOR EACH ROW
BEGIN
    SELECT SEQ_CODISESTADISTICS.NEXTVAL INTO :NEW.idCodi
    FROM DUAL;
END autoincrement_CODISESTADISTICS;
/
-------------------------------------------------------------------------------

