--------------------------------------------------------------------------------
--                  SCRIPT D'IMPLEMENTACIO DEL SISTEMA
--    Inclou la implementaci de tots els requisits funcionals del sistema:
--        Manteniment d'APLICACIONS, DESENVOLUPADORS, USUARIS i  DESCARREGUES 
--        Utilitats
--        Consultes
--        Disparadors del mdul Estadstic
--------------------------------------------------------------------------------


-------------------------------------------------------------------------
--                  PAQUET D'UTILITATS
-- Inclou les insercions de logs i el manteniment de la taula PERSONES
-- Els procediments d'alta i modificaci s'engloban en un de sol,
-- seleccionant la opci corresponent amb el primer parmetre d'entrada.
-- Tamb conte una declaracio de variables globals que seran utilitzades 
-- per els disparadors del modul estadstic.
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE Util IS

    --  Declaraci de variables globals utilitzades en el mdul estadstic.

    v_anydescarregaNova NUMBER;
    v_anydescarregaVella NUMBER;
    v_imeiNou descarrega.imei%TYPE;
    v_imeiVell descarrega.imei%TYPE;
    v_paisNou descarrega.pais%TYPE;
    v_paisVell descarrega.pais%TYPE;
    v_aplicacioNova descarrega.aplicacio%TYPE;
    v_aplicacioVella descarrega.aplicacio%TYPE;
    v_preuNou descarrega.preu%TYPE;
    v_preuVell descarrega.preu%TYPE;

  PROCEDURE InsertarLog (
    p_procediment IN VARCHAR2,
    p_parEntrada IN logsprocediments.parametresentrada%type,
    p_parSortida IN OUT NOCOPY VARCHAR2) ;
      
  PROCEDURE AM_Persona (
    p_Opcio IN CHAR,
    p_Identificador IN persona.idpersona%TYPE,
    p_nom IN persona.nom%TYPE,
    p_cognoms IN persona.cognoms%TYPE,
    p_adrea IN persona.adrea%TYPE,
    p_Poblacio IN persona.poblacio%TYPE, 
    p_Pais IN persona.pais%TYPE,
    p_telefon IN persona.telefon%TYPE,
    p_email IN persona.email%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Persona (
    p_Identificador IN persona.idpersona%TYPE,
    rsp OUT NOCOPY VARCHAR2);
      
END Util;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY Util IS
               
  v_log logsprocediments.parametresentrada%type;
  v_log_opcio VARCHAR2(12);
  OPCIO_INCORRECTE EXCEPTION;
  NO_TROBAT EXCEPTION;

----------------------------------------------------------------------------
--   Insercions en la taula de logs
----------------------------------------------------------------------------
  PROCEDURE InsertarLog (
    p_procediment IN VARCHAR2,
    p_parEntrada IN logsprocediments.parametresentrada%type,
    p_parSortida IN OUT NOCOPY VARCHAR2) IS
    
  v_resposta_log VARCHAR2(255);
  BEGIN
    BEGIN
      INSERT INTO logsprocediments( nomprocediment, parametresentrada, parametressortida) 
          VALUES (p_procediment, p_parEntrada, p_parSortida);   
      v_resposta_log:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN OTHERS THEN 
        v_resposta_log:='ERROR:'||SQLERRM;
            
    END;

    -- En cas de que hagi un error en la inserci a la taula de logs, es retorna
    -- un Warning amb el tipus d'error concatenat al parmetre de resposta del procediment
    IF v_resposta_log <> 'OK' THEN
      p_parSortida:=p_parSortida || '  - Warning: Taula logs no actualitzada: ' || v_resposta_log;
    END IF;
    
  END InsertarLog;
  
--------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions de la taula PERSONES
--------------------------------------------------------------------------------  
  PROCEDURE AM_Persona (
    p_Opcio IN CHAR,
    p_Identificador IN persona.idpersona%TYPE,
    p_Nom IN persona.nom%TYPE,
    p_Cognoms IN persona.cognoms%TYPE,
    p_Adrea IN persona.adrea%TYPE,
    p_Poblacio IN persona.poblacio%TYPE,
    p_Pais IN persona.pais%TYPE,
    p_Telefon IN persona.telefon%TYPE,
    p_Email IN persona.email%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO PERSONA (idpersona, nom, cognoms, adrea, telefon, poblacio, pais, email) 
            VALUES (p_Identificador, p_Nom, p_Cognoms, p_Adrea, p_Telefon, p_Poblacio, p_Pais, p_Email);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE PERSONA SET nom=p_Nom, cognoms=p_Cognoms, adrea=p_Adrea, 
                telefon=p_Telefon, poblacio=p_Poblacio, pais=p_Pais, email=p_Email
                WHERE idpersona=p_Identificador;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: La persona ja existeix');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: La persona no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Opci=' || p_opcio ||', IdPersona.=' || p_Identificador || ', Nom='|| p_Nom || ', Cognoms=' || p_Cognoms || 
            ', Adrea=' || p_Adrea || ', Poblaci=' || p_Poblacio || ', Pais=' || p_Pais ||', Tel.=' || p_Telefon || ', Email=' ||  p_Email); 
    InsertarLog('AM_Persona:' || v_log_opcio, v_log, rsp);
  
  END AM_Persona;
  
---------------------------------------------------------------------  
-- Procediment de baixa de Persones
-------------------------------------------------------------------    
  PROCEDURE Baixa_Persona (
    p_Identificador IN persona.idpersona%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM PERSONA WHERE idpersona=p_Identificador;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: La persona no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: La persona es un usuari, un desenvolupador o un representant legal. No es pot donar de baixa.';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;
   
    -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdPersona=' || p_Identificador);
    InsertarLog('Baixa_Persona', v_log, rsp);
    
  END Baixa_Persona;

END Util;
/

-------------------------------------------------------------------------
--      PAQUET DE MANTENIMENT D'APLICACIONS I ENTITATS ASSOCIADES
-- Els procediments d'alta i modificaci s'engloban en un de sol,
-- seleccionant la opci corresponent amb el primer parmetre d'entrada.
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE MantenimentAplicacions IS

  PROCEDURE AM_Aplicacio(
    p_Opcio IN CHAR,
    p_Identificador IN aplicacio.idaplicacio%TYPE,
    p_Versio IN aplicacio.versio%TYPE,
    p_Data IN aplicacio.datapujada%TYPE,
    p_Url IN aplicacio.url%TYPE,
    p_Resolucio IN aplicacio.resoluciomin%TYPE,
    p_Activa IN aplicacio.activa%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Aplicacio(
    p_Identificador IN aplicacio.idaplicacio%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE AM_SistSuportat(
    p_opcio IN CHAR,
    p_Aplicacio IN sistsuportat.aplicacio%TYPE,
    p_SistOperatiu IN sistsuportat.sistemaoperatiu%TYPE,
    p_MidaAplicacio IN sistsuportat.midaaplicacio%TYPE,
    p_FitxerBinari IN sistsuportat.fitxerbinari%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_SistSuportat(
    p_Aplicacio IN sistsuportat.aplicacio%TYPE,
    p_SistOperatiu IN sistsuportat.sistemaoperatiu%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE AM_Preu(
    p_opcio IN CHAR,
    p_Aplicacio IN aplpreu.aplicacio%TYPE,
    p_Pais IN aplpreu.pais%TYPE,
    p_Preu IN aplpreu.preu%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Preu(
    p_Aplicacio IN aplpreu.aplicacio%TYPE,
    p_Pais IN aplpreu.pais%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE AM_Descripcio(
    p_opcio IN CHAR,
    p_Aplicacio IN apldescrita.aplicacio%TYPE,
    p_Idioma IN apldescrita.idioma%TYPE,
    p_Descripcio IN apldescrita.descripcio%TYPE,
    rsp OUT NOCOPY VARCHAR2);
  
  PROCEDURE Baixa_Descripcio(
    p_Aplicacio IN apldescrita.aplicacio%TYPE,
    p_Idioma IN apldescrita.idioma%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE AB_Desenvolupada(
    p_opcio IN CHAR,
    p_Aplicacio IN apldesenvolupada.aplicacio%TYPE,
    p_Desenvolupador IN apldesenvolupada.desenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
END MantenimentAplicacions;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY MantenimentAplicacions IS

  v_log logsprocediments.parametresentrada%type;
  v_log_opcio VARCHAR2(12);
  resHoritzontal VARCHAR2(10);
  resVertical VARCHAR2(10);
  OPCIO_INCORRECTE EXCEPTION;
  NO_TROBAT EXCEPTION;
  FORMAT_RESOLUCIO_INCORRECTE EXCEPTION;
  
--------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions d'aplicacions
--------------------------------------------------------------------------------
  PROCEDURE AM_Aplicacio(
    p_Opcio IN CHAR,
    p_Identificador IN aplicacio.idaplicacio%TYPE,
    p_Versio IN aplicacio.versio%TYPE,
    p_Data IN aplicacio.datapujada%TYPE,
    p_Url IN aplicacio.url%TYPE,
    p_Resolucio IN aplicacio.resoluciomin%TYPE,
    p_Activa IN aplicacio.activa%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      resHoritzontal:=substr(p_Resolucio,1,instr(p_Resolucio,'x')-1);
      resVertical:= substr(p_Resolucio,instr(p_Resolucio,'x')+1,length(p_Resolucio)-instr(p_Resolucio,'x'));
       IF resHoritzontal IS NULL OR resVertical IS NULL THEN  
          RAISE FORMAT_RESOLUCIO_INCORRECTE;
      ELSE
          resHoritzontal:=translate(resHoritzontal,'T0123456789','T');
          resVertical:=translate(resVertical,'T0123456789','T');
          IF resHoritzontal IS NOT NULL OR resVertical IS NOT NULL THEN 
                RAISE FORMAT_RESOLUCIO_INCORRECTE;
          END IF;
      END IF;

      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO APLICACIO (idaplicacio, versio, datapujada, url, resoluciomin, activa) 
            VALUES (p_identificador, p_versio, p_data, p_url, p_resolucio, p_activa);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE APLICACIO SET versio=p_versio, datapujada=p_data, url=p_url, 
                resoluciomin=p_resolucio, activa=p_activa
                WHERE idaplicacio=p_identificador;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: L''aplicaci ja existeix');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''aplicaci no existeix.');
          ROLLBACK;
      WHEN FORMAT_RESOLUCIO_INCORRECTE THEN
          rsp:= ('ERROR: El format de resoluci no es correcte. Ha de ser ''DIGITSxDIGITS''- Exemple: 480x800');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Opci=' || p_opcio ||', IdAp.=' || p_identificador || ', Vers.='|| p_versio || ', Data=' || p_data || 
            ', Url=' || p_url || ', Res.Min=' || p_resolucio || ', Activa=' ||  p_activa); 
    Util.InsertarLog('AM_Aplicacio:' || v_log_opcio, v_log, rsp);
  
  END AM_Aplicacio;

---------------------------------------------------------------------  
-- Procediment de baixas d'aplicacions
-------------------------------------------------------------------    
  PROCEDURE Baixa_Aplicacio(
    p_Identificador IN aplicacio.idaplicacio%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM APLICACIO WHERE idaplicacio=p_identificador;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''aplicaci no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: S''han fet descrregues d''aquesta aplicaci. No es pot donar de baixa.';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
    END;
   
    -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdAp.=' || p_identificador);
    Util.InsertarLog('Baixa_Aplicacio', v_log, rsp);
    
  END Baixa_Aplicacio;

--------------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions de sistemes operatius suportats per aplicacions
--------------------------------------------------------------------------------------
  PROCEDURE AM_SistSuportat(
    p_opcio IN CHAR,
    p_Aplicacio IN sistsuportat.aplicacio%TYPE,
    p_SistOperatiu IN sistsuportat.sistemaoperatiu%TYPE,
    p_MidaAplicacio IN sistsuportat.midaaplicacio%TYPE,
    p_FitxerBinari IN sistsuportat.fitxerbinari%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO SISTSUPORTAT (aplicacio, sistemaoperatiu, midaaplicacio, fitxerbinari) 
            VALUES (p_Aplicacio, p_SistOperatiu, p_MidaAplicacio, p_FitxerBinari);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE SISTSUPORTAT SET midaaplicacio=p_MidaAplicacio, fitxerbinari=p_FitxerBinari
                WHERE aplicacio=p_Aplicacio AND sistemaoperatiu=p_SistOperatiu;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: L''aplicaci ja suporta aquest sistema operatiu');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''aplicaci i el sist. operatiu no estan relacionats o no existeixen.');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2291 THEN
            rsp:='ERROR: L''aplicacio i/o el sist. operatiu no existeixen';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments
      -------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdAp.=' || p_Aplicacio || ', SistOp.=' || p_SistOperatiu || 
              ', Mida=' ||  p_MidaAplicacio || ', Fitxer=' || p_FitxerBinari); 
      Util.InsertarLog('AM_SistSuportat:' || v_log_opcio, v_log, rsp);

  END AM_SistSuportat;
    
--------------------------------------------------------------------------------  
-- Procediment de baixa de sistemes operatius suportats per aplicacions
--------------------------------------------------------------------------------
  PROCEDURE Baixa_SistSuportat(
    p_Aplicacio IN sistsuportat.aplicacio%TYPE,
    p_SistOperatiu IN sistsuportat.sistemaoperatiu%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      DELETE FROM SISTSUPORTAT WHERE aplicacio=p_Aplicacio AND sistemaoperatiu=p_SistOperatiu;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:='ERROR: L''aplicaci i el sist. operatiu no estan relacionats o no existeixen';
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;
  
      -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdAp.=' || p_Aplicacio || ', SistOp.=' || p_SistOperatiu);
    Util.InsertarLog('Baixa_SistSuportat', v_log, rsp);  
  
  END Baixa_SistSuportat;  
    
--------------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions dels preus de les aplicacions
--------------------------------------------------------------------------------------
  PROCEDURE AM_Preu(
    p_opcio IN CHAR,
    p_Aplicacio IN aplpreu.aplicacio%TYPE,
    p_Pais IN aplpreu.pais%TYPE,
    p_Preu IN aplpreu.preu%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO APLPREU (aplicacio, pais, preu) 
            VALUES (p_Aplicacio, p_Pais, p_Preu);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE APLPREU SET preu=p_Preu
                WHERE aplicacio=p_Aplicacio AND pais=p_Pais;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: L''aplicaci ja t un preu fixat per aquest pas');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''aplicaci no te un preu per aquest pas o no existeix el pas i/o l''aplicaci');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2291 THEN
            rsp:='ERROR: L''aplicaci i/o el pas no existeixen';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments
      -------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdAp.=' || p_Aplicacio || ', Pais=' || p_Pais || 
              ', Preu=' ||  p_Preu); 
      Util.InsertarLog('AM_Preu:' || v_log_opcio, v_log, rsp);

  END AM_Preu;

--------------------------------------------------------------------------------------  
-- Procediment de baixa de preus de les aplicacions
--------------------------------------------------------------------------------------
  PROCEDURE Baixa_Preu(
    p_Aplicacio IN aplpreu.aplicacio%TYPE,
    p_Pais IN aplpreu.pais%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      DELETE FROM APLPREU WHERE aplicacio=p_Aplicacio AND pais=p_Pais;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:='ERROR: L''aplicaci no te un preu per aquest pas o no existeix el pas i/o l''aplicaci';
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;
  
      -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdAp.=' || p_Aplicacio || ', Pais=' || p_Pais);
    Util.InsertarLog('Baixa_Preu', v_log, rsp);  
  
  END Baixa_Preu;  

--------------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions de les descripcions d'aplicacions
--------------------------------------------------------------------------------------
  PROCEDURE AM_Descripcio(
    p_opcio IN CHAR,
    p_Aplicacio IN apldescrita.aplicacio%TYPE,
    p_Idioma IN apldescrita.idioma%TYPE,
    p_Descripcio IN apldescrita.descripcio%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO APLDESCRITA (aplicacio, idioma, descripcio) 
            VALUES (p_Aplicacio, p_Idioma, p_Descripcio);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE APLDESCRITA SET descripcio=p_Descripcio
                WHERE aplicacio=p_Aplicacio AND idioma=p_idioma;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: L''aplicaci ja est descrita en aquest idioma');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''aplicaci no est descrita en aquest idioma o no existeix l''idioma i/o l''aplicaci');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2291 THEN
            rsp:='ERROR: L''aplicaci i/o l''idioma no existeixen';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments. 
      -- El parmetre d'entrada 'p_Descripci es trunca a les primeres 30 posicions
      -- ja que en cas contrari podria arrivar fins a 1024 bytes i no es pot emmagatzemar-ho tot.
      ---------------------------------------------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdAp.=' || p_Aplicacio || ', Idioma=' || p_Idioma || 
              ', Descripcio=' ||  SUBSTR(p_Descripcio,1,30) || '...'); 
      Util.InsertarLog('AM_Descripcio:' || v_log_opcio, v_log, rsp);

  END AM_Descripcio;

 --------------------------------------------------------------------------------------  
-- Procediment de baixa de descripci de les aplicacions
-------------------------------------------------------------------------------------- 
  PROCEDURE Baixa_Descripcio(
    p_Aplicacio IN apldescrita.aplicacio%TYPE,
    p_Idioma IN apldescrita.idioma%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM APLDESCRITA WHERE aplicacio=p_Aplicacio AND idioma=p_Idioma;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:='ERROR: L''aplicaci no est descrita en aquest idioma o no existeix l''idioma i/o l''aplicaci';
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;
  
      -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdAp.=' || p_Aplicacio || ', Idioma=' || p_Idioma);
    Util.InsertarLog('Baixa_Descripcio', v_log, rsp);  
  
  END Baixa_Descripcio;  

--------------------------------------------------------------------------------------  
-- Procediment d'altes i baixes dels desenvolupadors d'una aplicaci.
-- No es considera la possibilitat de modificaci ja que aquesta
-- entitat no t cap atribut, llevat dels que configuran la clau primaria.
--------------------------------------------------------------------------------------
  PROCEDURE AB_Desenvolupada(
    p_opcio IN CHAR,
    p_Aplicacio IN apldesenvolupada.aplicacio%TYPE,
    p_Desenvolupador IN apldesenvolupada.desenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO APLDESENVOLUPADA (aplicacio, desenvolupador) 
            VALUES (p_Aplicacio, p_Desenvolupador);   

        WHEN 'B' THEN
            v_log_opcio:='Baixa';
            DELETE FROM APLDESENVOLUPADA WHERE aplicacio=p_Aplicacio 
                    AND desenvolupador=p_Desenvolupador;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: Aquesta persona ja est registrada com a desenvolupador d''aquesta aplicaci');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o B=Baixa');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: Aquesta persona no ha desenvolupat l''aplicaci o no existeix el desenvolupador i/o l''aplicaci');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2291 THEN
            rsp:='ERROR: L''aplicaci i/o el desenvolupador no existeixen';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments. 
      ---------------------------------------------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdAp.=' || p_Aplicacio || ', Desenvolupador=' || p_Desenvolupador); 
      Util.InsertarLog('AB_Desenvolupada:' || v_log_opcio, v_log, rsp);

  END AB_Desenvolupada;
  
    
END MantenimentAplicacions;
/


-------------------------------------------------------------------------
--  PAQUET DE MANTENIMENT DELS DESENVOLUPADORS I ENTITATS ASSOCIADES
-- Els procediments d'alta i modificaci s'engloban en un de sol,
-- seleccionant la opci corresponent amb el primer parmetre d'entrada.
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE MantenimentDesenvolupadors IS

  PROCEDURE AM_Empresa(
    p_Opcio IN CHAR,
    p_IdEmpresa IN empresa.idempresa%TYPE,
    p_PaisRegistre IN empresa.paisregistre%TYPE,
    p_Representant IN empresa.representant%TYPE,
    p_Adrea IN empresa.adrea%TYPE,
    p_Poblacio IN empresa.poblacio%TYPE,
    p_Telefon IN empresa.telefon%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Empresa(
    p_IdEmpresa IN empresa.idempresa%TYPE,
    rsp OUT NOCOPY VARCHAR2);
   
  PROCEDURE AM_Desenvolupador(
    p_opcio IN CHAR,
    p_IdDesenvolupador IN desenvolupador.iddesenvolupador%TYPE,
    p_Empresa IN desenvolupador.empresa%TYPE,
    p_CodiDesenvolupador IN desenvolupador.codidesenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Desenvolupador(
    p_IdDesenvolupador IN desenvolupador.iddesenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2);
   
END MantenimentDesenvolupadors;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY MantenimentDesenvolupadors IS

  v_log logsprocediments.parametresentrada%type;
  v_log_opcio VARCHAR2(12);
  OPCIO_INCORRECTE EXCEPTION;
  NO_TROBAT EXCEPTION;
  
--------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions d'empreses
--------------------------------------------------------------------------------
  PROCEDURE AM_Empresa(
    p_Opcio IN CHAR,
    p_IdEmpresa IN empresa.idempresa%TYPE,
    p_PaisRegistre IN empresa.paisregistre%TYPE,
    p_Representant IN empresa.representant%TYPE,
    p_Adrea IN empresa.adrea%TYPE,
    p_Poblacio IN empresa.poblacio%TYPE,
    p_Telefon IN empresa.telefon%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO EMPRESA (idempresa, paisregistre, representant, adrea, poblacio, telefon) 
            VALUES (p_IdEmpresa, p_PaisRegistre, p_Representant, p_Adrea, p_Poblacio, p_Telefon);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE EMPRESA SET paisregistre=p_PaisRegistre, representant=p_Representant, 
                adrea=p_Adrea, telefon=p_Telefon
                WHERE idempresa=p_IdEmpresa;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: L''empresa ja existeix');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''empresa no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Opci=' || p_opcio ||', IdEmp.=' || p_IdEmpresa || ', Pais='|| p_PaisRegistre || 
            ', Rep.Legal' || p_Representant || ', Adrea=' || p_Adrea || ', Tel.=' ||  p_Telefon); 
    Util.InsertarLog('AM_Empresa:' || v_log_opcio, v_log, rsp);
  
  END AM_Empresa;

---------------------------------------------------------------------  
-- Procediment de baixas d'empreses
-------------------------------------------------------------------    
  PROCEDURE Baixa_Empresa(
    p_IdEmpresa IN empresa.idempresa%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM EMPRESA WHERE idempresa=p_IdEmpresa;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''empresa no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: L''empresa te desenvolupadors. No es pot donar de baixa';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;
   
    -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdEmp.=' || p_IdEmpresa);
    Util.InsertarLog('Baixa_Empresa', v_log, rsp);
    
  END Baixa_Empresa;

--------------------------------------------------------------------------------------  
-- Procediment d'alta i modificaci dels desenvolupadors d'aplicacions
--------------------------------------------------------------------------------------
  PROCEDURE AM_Desenvolupador(
    p_opcio IN CHAR,
    p_IdDesenvolupador IN desenvolupador.iddesenvolupador%TYPE,
    p_Empresa IN desenvolupador.empresa%TYPE,
    p_CodiDesenvolupador IN desenvolupador.codidesenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO DESENVOLUPADOR (iddesenvolupador, empresa, codidesenvolupador) 
            VALUES (p_IdDesenvolupador, p_Empresa, p_CodiDesenvolupador);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE DESENVOLUPADOR SET empresa=p_Empresa, codidesenvolupador=p_CodiDesenvolupador
                WHERE iddesenvolupador=p_IdDesenvolupador;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          IF v_log_opcio='Alta' THEN
            rsp:= ('ERROR: El desenvolupador i/o el codi del desenvolupador ja existeixen');
          ELSE
            rsp:= ('ERROR: El codi del desenvolupador ja existeix');
          END IF;
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: El desenvolupador no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments
      -------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdDesenv.=' || p_IdDesenvolupador || 
              ', Empresa=' ||  p_Empresa || ', Codi=' || p_CodiDesenvolupador); 
      Util.InsertarLog('AM_Desenvolupador:' || v_log_opcio, v_log, rsp);

  END AM_Desenvolupador;
    
--------------------------------------------------------------------------------  
-- Procediment de baixa de desenvolupadors d'aplicacions
--------------------------------------------------------------------------------
  PROCEDURE Baixa_Desenvolupador(
    p_IdDesenvolupador IN desenvolupador.iddesenvolupador%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      DELETE FROM DESENVOLUPADOR WHERE iddesenvolupador=p_IdDesenvolupador;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:='ERROR: El desenvolupador no existeix.';
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: El desenvolupador te aplicacions desenvolupades. No es pot donar de baixa';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;
  
      -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdDesenv.=' || p_IdDesenvolupador);
    Util.InsertarLog('Baixa_Desenvolupador', v_log, rsp);  
  
  END Baixa_Desenvolupador;  
  
END MantenimentDesenvolupadors;
/


-------------------------------------------------------------------------
--  PAQUET DE MANTENIMENT DELS USUARIS I ENTITATS ASSOCIADES
-- Els procediments d'alta i modificaci s'engloban en un de sol,
-- seleccionant la opci corresponent amb el primer parmetre d'entrada.
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE MantenimentUsuaris IS

  PROCEDURE AM_Dispositiu(
    p_Opcio IN CHAR,
    p_Imei IN dispositiu.imei%TYPE,
    p_IdPropietari IN dispositiu.idpropietari%TYPE,
    p_SoDispositiu IN dispositiu.sodispositiu%TYPE,
    p_ModelDisp IN dispositiu.modeldisp%TYPE,
    p_Resolucio IN dispositiu.resolucio%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Dispositiu(
    p_Imei IN dispositiu.imei%TYPE,
    rsp OUT NOCOPY VARCHAR2);
   
  PROCEDURE AM_Usuari(
    p_opcio IN CHAR,
    p_IdUsuari IN usuari.idusuari%TYPE,
    p_Operador IN usuari.operadorusuari%TYPE,
    p_NumeroMobil IN usuari.numeromobil%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa_Usuari(
    p_IdUsuari IN usuari.idusuari%TYPE,
    rsp OUT NOCOPY VARCHAR2);
   
END MantenimentUsuaris;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY MantenimentUsuaris IS

  v_log logsprocediments.parametresentrada%type;
  v_log_opcio VARCHAR2(12);
  resHoritzontal VARCHAR2(10);
  resVertical VARCHAR2(10);
  v_propietari dispositiu.idpropietari%TYPE;
  v_contador NUMBER;
  OPCIO_INCORRECTE EXCEPTION;
  NO_TROBAT EXCEPTION;
  FORMAT_RESOLUCIO_INCORRECTE EXCEPTION;
  MOD_INCORRECTA_PROPIETARI EXCEPTION;
  
  
--------------------------------------------------------------------------------  
-- Procediment d'altes i modificacions de dispositius mbils
--------------------------------------------------------------------------------
  PROCEDURE AM_Dispositiu(
    p_Opcio IN CHAR,
    p_Imei IN dispositiu.imei%TYPE,
    p_IdPropietari IN dispositiu.idpropietari%TYPE,
    p_SoDispositiu IN dispositiu.sodispositiu%TYPE,
    p_ModelDisp IN dispositiu.modeldisp%TYPE,
    p_Resolucio IN dispositiu.resolucio%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      resHoritzontal:=substr(p_Resolucio,1,instr(p_Resolucio,'x')-1);
      resVertical:= substr(p_Resolucio,instr(p_Resolucio,'x')+1,length(p_Resolucio)-instr(p_Resolucio,'x'));
       IF resHoritzontal IS NULL OR resVertical IS NULL THEN  
          RAISE FORMAT_RESOLUCIO_INCORRECTE;
      ELSE
          resHoritzontal:=translate(resHoritzontal,'T0123456789','T');
          resVertical:=translate(resVertical,'T0123456789','T');
          IF resHoritzontal IS NOT NULL OR resVertical IS NOT NULL THEN 
                RAISE FORMAT_RESOLUCIO_INCORRECTE;
          END IF;
      END IF;

      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO DISPOSITIU (imei, idpropietari, sodispositiu, modeldisp, resolucio) 
            VALUES (p_Imei, p_IdPropietari, p_SoDispositiu, p_ModelDisp, p_Resolucio);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            
            -- Control per no modificar el propietari del dispositiu si t alguna descrrega feta amb aquest dispositiu.
            SELECT idpropietari INTO v_propietari FROM DISPOSITIU WHERE imei=p_Imei;
            IF p_IdPropietari<> v_propietari THEN
              SELECT COUNT (*) INTO v_contador FROM DESCARREGA WHERE imei=p_Imei;
              IF v_contador > 0 THEN
                RAISE MOD_INCORRECTA_PROPIETARI;
              END IF;
            END IF;
            
            UPDATE DISPOSITIU SET idpropietari=p_IdPropietari, sodispositiu=p_SoDispositiu, 
                modeldisp=p_ModelDisp, resolucio=p_Resolucio
                WHERE imei=p_Imei;
            
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          rsp:= ('ERROR: El dispositiu ja existeix');
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_DATA_FOUND THEN
          rsp:= ('ERROR: El dispositiu no existeix.');
          ROLLBACK;
      WHEN MOD_INCORRECTA_PROPIETARI THEN
          rsp:= ('ERROR: Aquest dispositiu t descrregues. No es pot modificar el propietari.');
          ROLLBACK;
      WHEN FORMAT_RESOLUCIO_INCORRECTE THEN
          rsp:= ('ERROR: El format de resoluci no es correcte. Ha de ser ''DIGITSxDIGITS''- Exemple: 480x800');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Opci=' || p_opcio ||', Imei=' || p_Imei || ', Propietari='|| p_IdPropietari || 
            ', SoDisp.' || p_SoDispositiu || ', Model=' || p_ModelDisp || ', Resoluci=' ||  p_Resolucio); 
    Util.InsertarLog('AM_Dispositiu:' || v_log_opcio, v_log, rsp);
  
  END AM_Dispositiu;

---------------------------------------------------------------------  
-- Procediment de baixas de dispositius mbils
-------------------------------------------------------------------    
  PROCEDURE Baixa_Dispositiu(
    p_Imei IN dispositiu.imei%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM DISPOSITIU WHERE imei=p_Imei;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: El dispositiu no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: S''han fet descrregues amb aquest dispositiu. No es pot donar de baixa.';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
    END;
   
    -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('Imei=' || p_Imei);
    Util.InsertarLog('Baixa_Dispositiu', v_log, rsp);
    
  END Baixa_Dispositiu;

--------------------------------------------------------------------------------------  
-- Procediment d'alta i modificaci d'usuaris 
--------------------------------------------------------------------------------------
  PROCEDURE AM_Usuari(
    p_opcio IN CHAR,
    p_IdUsuari IN usuari.idusuari%TYPE,
    p_Operador IN usuari.operadorusuari%TYPE,
    p_NumeroMobil IN usuari.numeromobil%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      CASE UPPER(p_Opcio)
        WHEN 'A' THEN
            v_log_opcio:='Alta';
            INSERT INTO USUARI (idusuari, operadorusuari, numeromobil) 
            VALUES (p_IdUsuari, p_Operador, p_NumeroMobil);   

        WHEN 'M' THEN
            v_log_opcio:='Modificaci';
            UPDATE USUARI SET operadorusuari=p_Operador, numeromobil=p_NumeroMobil
                WHERE idusuari=p_IdUsuari;
            IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
            END IF;
        ELSE 
          v_log_opcio:=NULL;
          RAISE OPCIO_INCORRECTE;
      END CASE;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN DUP_VAL_ON_INDEX THEN
          IF v_log_opcio='Alta' THEN
            rsp:= ('ERROR: L''usuari i/o el nmero de mbil ja existeixen');
          ELSE
            rsp:= ('ERROR: El nmero de mbil ja existeix');
          END IF;
          ROLLBACK; 
      WHEN OPCIO_INCORRECTE THEN
          rsp:= ('ERROR: L''opci de manteniment ha de ser A=Alta o M=Modificaci');
          ROLLBACK;
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: L''usuari no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments
      -------------------------------------------------------
      v_log:=('Opci=' || p_opcio ||', IdUsuari=' || p_IdUsuari || 
              ', Operador=' || p_Operador || ', Mbil=' || p_NumeroMobil); 
      Util.InsertarLog('AM_Usuari:' || v_log_opcio, v_log, rsp);

  END AM_Usuari;
    
--------------------------------------------------------------------------------  
-- Procediment de baixa d'usuaris
--------------------------------------------------------------------------------
  PROCEDURE Baixa_Usuari(
    p_IdUsuari IN usuari.idusuari%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      DELETE FROM USUARI WHERE idusuari=p_IdUsuari;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:='ERROR: L''usuari no existeix.';
          ROLLBACK;
      WHEN OTHERS THEN 
          IF SQLCODE=-2292 THEN
            rsp:='ERROR: L''usuari t dispositius mbils. No es pot donar de baixa';
          ELSE
            rsp:='ERROR:'||SQLERRM;
          END IF;
          ROLLBACK;
    END;
  
      -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_log:=('IdUsuari=' || p_IdUsuari);
    Util.InsertarLog('Baixa_Usuari', v_log, rsp);  
  
  END Baixa_Usuari;  
  
END MantenimentUsuaris;
/


-------------------------------------------------------------------------
--  PAQUET DE MANTENIMENT DE LES DESCARREGES REATLITZADES PELS USUARIS
--
--    Es crean dos procediments d'alta sobrecarregats:  Un a partir dels tres
--  parmetres indispensables per poder emmagatzemar la informaci: 
--  L'Imei del dispositiu de descrrega, l'aplicaci descarregada i el mode de pagament.
--  La resta de dades associades s'extreuen de la mateixa base de dades.
--
--    L'altre procediment d'alta,  permet introduir totes les dades d'una 
--  descrrega en cas de que es necesiti emmagatzemar una informaci diferent a la 
--  que hi ha en aquell moment a la base de dades. Per exemple una data diferent 
--  de la del sistema, un preu diferent al referenciat en la taula APLPREU o un
--  operador o sistema operatiu diferent al que te l'usuari en aquell moment.
--
--    Tamb es crea un proces de modificaci i un altre de baixa en base al 
--  identicador de la descrrega. Aquest identificador s'ha retornat com a
--  parmetre de sortida en el moment de l'alta.
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE MantenimentDescarregues IS

  PROCEDURE Alta(
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    p_Identificador OUT NOCOPY descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Alta(
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_Pais IN descarrega.pais%TYPE,
    p_NomOperador IN descarrega.nomoperador%TYPE,
    p_Data IN descarrega.datadescarrega%TYPE,
    p_Preu IN descarrega.preu%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    p_Identificador OUT NOCOPY descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Modificacio(
    p_Identificador IN descarrega.iddescarrega%TYPE,
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_Pais IN descarrega.pais%TYPE,
    p_NomOperador IN descarrega.nomoperador%TYPE,
    p_Data IN descarrega.datadescarrega%TYPE,
    p_Preu IN descarrega.preu%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Baixa(
    p_Identificador IN descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2);
   
END MantenimentDescarregues;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY MantenimentDescarregues IS

  v_logEnt logsprocediments.parametresentrada%type;
  v_logSor logsprocediments.parametresentrada%type;
  APL_PREU_INCORRECTE EXCEPTION;
  NO_TROBAT EXCEPTION;
  SIST_SUPORTAT_INCORRECTE EXCEPTION;
  APLICACIO_NO_ACTIVA EXCEPTION;
  RESOLUCIO_NO_SUPORTADA EXCEPTION;
  
--------------------------------------------------------------------------------  
-- Procediment d'alta d'una descrrega amb valors existents a la base de dades.
--------------------------------------------------------------------------------
  PROCEDURE Alta(
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    p_Identificador OUT NOCOPY descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS

    -- Declaracio de variables
    ----------------------------------------------------------------------------
    v_Pais descarrega.pais%TYPE;
    v_NomOperador descarrega.nomoperador%TYPE;
    v_Preu descarrega.preu%TYPE;
    v_Usuari dispositiu.idPropietari%TYPE;
    v_SisOperatiu dispositiu.sodispositiu%TYPE;
    v_resolucio_disp dispositiu.resolucio%TYPE;
    v_resolucio_app aplicacio.resoluciomin%TYPE;
    v_contador NUMBER;
    
    v_res_horitzontal_dis NUMBER;
    v_res_vertical_dis NUMBER;
    v_res_horitzontal_app NUMBER;
    v_res_vertical_app NUMBER;

  BEGIN
    BEGIN
 
      SELECT idpropietari, sodispositiu, resolucio INTO v_Usuari, v_SisOperatiu, v_resolucio_disp 
      FROM DISPOSITIU WHERE imei=p_Imei;

     -- Control per veure si l'aplicaci suporta el sistema operatiu del dispositiu
      SELECT COUNT(*) INTO v_contador FROM SISTSUPORTAT WHERE aplicacio=p_Aplicacio AND sistemaoperatiu=v_SisOperatiu;
      IF v_contador = 0 THEN  RAISE SIST_SUPORTAT_INCORRECTE;
      END IF;
      
      SELECT operadorusuari INTO v_NomOperador FROM USUARI  WHERE idusuari=v_Usuari;
      SELECT pais INTO v_Pais FROM PERSONA WHERE idpersona=v_Usuari;

      -- Control per veure si l'aplicaci te un preu associat al pais de registre del usuari   
      SELECT COUNT(*) INTO v_contador FROM APLPREU WHERE aplicacio=p_Aplicacio AND pais=v_Pais;
      IF v_contador = 0 THEN RAISE APL_PREU_INCORRECTE;
      END IF;
      
      -- Control per comprovar si l'aplicaci esta activa per a descrrega      
      SELECT activa, resoluciomin INTO v_contador, v_resolucio_app FROM APLICACIO WHERE idaplicacio=p_aplicacio;
      IF v_contador = 0 THEN RAISE APLICACIO_NO_ACTIVA;
      END IF;
            
      -- Control per comprovar que el dispositiu suporti la resoluci de l'aplicaci.
      v_res_horitzontal_dis:=substr(v_resolucio_disp,1,instr(v_resolucio_disp,'x')-1);
      v_res_vertical_dis:=substr(v_resolucio_disp,instr(v_resolucio_disp,'x')+1,length(v_resolucio_disp)-instr(v_resolucio_disp,'x'));
      v_res_horitzontal_app:=substr(v_resolucio_app,1,instr(v_resolucio_app,'x')-1);
      v_res_vertical_app:=substr(v_resolucio_app,instr(v_resolucio_app,'x')+1,length(v_resolucio_app)-instr(v_resolucio_app,'x'));
  
      IF v_res_horitzontal_dis < v_res_horitzontal_app OR v_res_vertical_dis < v_res_vertical_app THEN
        IF v_res_horitzontal_dis < v_res_vertical_app OR v_res_vertical_dis < v_res_horitzontal_app
          THEN RAISE RESOLUCIO_NO_SUPORTADA;
        END IF;
      END IF;

      SELECT preu INTO v_preu FROM APLPREU WHERE aplicacio=p_Aplicacio AND pais=v_Pais;
           
      INSERT INTO DESCARREGA (aplicacio, imei, pais, nomoperador, datadescarrega, preu, modepagament) 
        VALUES (p_Aplicacio, p_Imei, v_Pais, v_NomOperador, SYSDATE, v_Preu, p_ModePagament)
        RETURNING iddescarrega INTO p_Identificador; 

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN APL_PREU_INCORRECTE THEN
          rsp:= ('ERROR: L''aplicaci no existeix o no te un preu associat al pais de registre del usuari');
          ROLLBACK;
      WHEN SIST_SUPORTAT_INCORRECTE THEN
          rsp:= ('ERROR: L''aplicaci no existeix o no suporta el sistema operatiu del dispositiu de descrrega');
          ROLLBACK;    
      WHEN NO_DATA_FOUND THEN
          rsp:= ('ERROR: El dispositiu de descrrega no existeix.');
          ROLLBACK;
      WHEN APLICACIO_NO_ACTIVA THEN
          rsp:= ('ERROR: L''aplicaci no esta activa per a descrrega.');
          ROLLBACK;
      WHEN RESOLUCIO_NO_SUPORTADA THEN
          rsp:= ('ERROR: El dispositiu no suporta la resoluci mnima de l''aplicaci');
          ROLLBACK;       
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_logEnt:=('Aplicaci=' || p_Aplicacio ||', Imei=' || p_Imei || ', Mode Pag.='|| p_ModePagament); 
    v_logSor:=('IdDescarrega=' || p_Identificador || ', Resposta= ' || rsp);
    Util.InsertarLog('Alta Descrrega', v_logEnt, v_logSor );
  
  END Alta;
  
--------------------------------------------------------------------------------  
-- Procediment d'alta manual d'una descrrega.
-- No es tenen en compte el valors actuals del usuari en quan a operador,
-- sistema operatiu o preu de l'aplicaci en  el pas de registre del usuari. 
-- Es comproven per les restriccions referencials corresponents dels atributs
-- Aplicacio, Imei, Pas i Operador
--------------------------------------------------------------------------------  
  PROCEDURE Alta(
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_Pais IN descarrega.pais%TYPE,
    p_NomOperador IN descarrega.nomoperador%TYPE,
    p_Data IN descarrega.datadescarrega%TYPE,
    p_Preu IN descarrega.preu%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    p_Identificador OUT NOCOPY descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      INSERT INTO DESCARREGA (aplicacio, imei, pais, nomoperador, datadescarrega, preu, modepagament) 
        VALUES (p_Aplicacio, p_Imei, p_Pais, p_NomOperador, p_Data, p_Preu, p_ModePagament)
        RETURNING iddescarrega INTO p_Identificador; 

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_logEnt:=('Apl=' || p_Aplicacio ||', Imei=' || p_Imei || ', Pas='|| p_Pais ||', Operador='|| p_NomOperador ||
               ', Data='|| p_Data || ', Preu='|| p_Preu ||', M.Pag.='|| p_ModePagament);  
    v_logSor:=('IdDescarrega=' || p_Identificador || ', Resposta= ' || rsp);
    Util.InsertarLog('Alta Descrrega', v_logEnt, v_logSor );
  
  END Alta;    

--------------------------------------------------------------------------------------  
-- Procediment de modificaci de descrregues fetes pels usuaris.
-- Igual que en el procediment d'alta anterior, ne es t en compte el valors 
-- actuals del usuari en quan a operador, sistema operatiu o preu de l'aplicaci
-- en  el pas de registre del usuari. 
-- Es comproven igualment les restriccions referencials corresponents dels atributs
-- Aplicacio, Imei, Pas i Operador.
--------------------------------------------------------------------------------------
  PROCEDURE Modificacio(
    p_Identificador IN descarrega.iddescarrega%TYPE,
    p_Aplicacio IN descarrega.aplicacio%TYPE,
    p_Imei IN descarrega.imei%TYPE,
    p_Pais IN descarrega.pais%TYPE,
    p_NomOperador IN descarrega.nomoperador%TYPE,
    p_Data IN descarrega.datadescarrega%TYPE,
    p_Preu IN descarrega.preu%TYPE,
    p_ModePagament IN descarrega.modepagament%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      UPDATE DESCARREGA SET aplicacio=p_Aplicacio, imei=p_Imei, pais=p_Pais, nomoperador=p_NomOperador,
                datadescarrega=p_Data, preu=p_Preu, modepagament=p_ModePagament
        WHERE iddescarrega=p_Identificador;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;

      rsp:='OK'; 
      COMMIT;

    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: La descrrega no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;

      -- Inserci de control a la taula LosgsProcediments
      -------------------------------------------------------
      v_logEnt:=('IdDesc.=' || p_Identificador || ', Aplicaci=' ||  p_Aplicacio ||
              ', Imei=' || p_Imei || ', Pas=' || p_Pais || ', Operador=' ||  p_NomOperador ||
              ', DataDesc.=' || p_Data || ', Preu=' || p_Preu || ', ModePag.=' || p_ModePagament); 
      Util.InsertarLog('Modificacio', v_logEnt, rsp);

  END Modificacio;

---------------------------------------------------------------------  
-- Procediment de baixa de descrregues fetes pels usuaris
-------------------------------------------------------------------    
  PROCEDURE Baixa(
    p_Identificador IN descarrega.iddescarrega%TYPE,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      DELETE FROM DESCARREGA WHERE iddescarrega=p_Identificador;
      IF SQL%ROWCOUNT = 0 THEN RAISE NO_TROBAT;
      END IF;
      rsp:='OK'; 
      COMMIT;
      
    EXCEPTION
      WHEN NO_TROBAT THEN
          rsp:= ('ERROR: La descrrega no existeix.');
          ROLLBACK;
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
          ROLLBACK;
    END;
   
    -- Inserci de control a la taula LosgsProcediments
    ------------------------------------------------------
    v_logEnt:=('Identificador Desc.=' || p_Identificador);
    Util.InsertarLog('Baixa Descrrega', v_logEnt, rsp);
    
  END Baixa;
    
END MantenimentDescarregues;
/

-------------------------------------------------------------------------
--          PAQUET DE PROCEDIMENTS DE CONSULTA
-- Tots els procediments tenen un parmetre de sortida que es una variable
-- REF CURSOR. Aquesta variable s'utilitzara per retornar la consulta demanada
-------------------------------------------------------------------------
--           CAPALERA DEL PAQUET  (Secci declarativa pblica)
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE Consultes IS

  TYPE t_RefCur IS REF CURSOR;

  PROCEDURE Desenvolupadors_Pais(
    p_Pais IN descarrega.pais%TYPE,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Aplicacions_Actives(
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Paisos_Descarrega(
    p_aplicacio IN aplicacio.idaplicacio%TYPE,
    p_any IN NUMBER,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Activitat_Usuari(
    p_telefon IN usuari.numeromobil%TYPE,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2);
    
  PROCEDURE Usuaris_DinersGastats(
    p_any IN NUMBER,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2);
    
END Consultes;
/

-------------------------------------------------------------------------
--                COS DEL PAQUET
-------------------------------------------------------------------------
CREATE OR REPLACE PACKAGE BODY Consultes IS

  v_log logsprocediments.parametresentrada%type;
  
------------------------------------------------------------------------------------ 
--     CONSULTA DE DESENVOLUPADORS D'UN PAS DONAT
-- Retorna un cursor apuntant a la llista de desenvolupadors d'una pas donat, 
-- amb totes les seves dades, incloent el nmero d'aplicacions diferents publicades.
------------------------------------------------------------------------------------
  PROCEDURE Desenvolupadors_Pais(
    p_Pais IN descarrega.pais%TYPE,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      -------  Obertura del cursor sobre el select de la consulta -----------------
      OPEN p_cursor FOR 
        SELECT pe.nom, pe.cognoms, de.codidesenvolupador, de.empresa, count(*) as numapps, pe.adrea, pe.poblacio, pe.telefon, pe.email 
        FROM (desenvolupador de JOIN persona pe ON de.iddesenvolupador=pe.idpersona) 
              JOIN apldesenvolupada ap ON ap.desenvolupador=de.iddesenvolupador
        WHERE pe.pais=p_Pais
        GROUP BY pe.nom, pe.cognoms, pe.adrea, pe.poblacio, pe.telefon, pe.email, de.codidesenvolupador, de.empresa;
                  
      rsp:='OK'; 
    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Pais=' || p_Pais);  
    rsp:='Dades dels desenvolupadors del pais: ' || p_Pais || ' i num. d''apps. desenvolupades.  RSP:' || rsp;
    Util.InsertarLog('Desenvolupadors_Pais', v_log, rsp); 
  
  END Desenvolupadors_Pais;

------------------------------------------------------------------------------------ 
--      CONSULTA D'APLICACIONS ACTIVES
-- Retorna un cursor apuntant a la llista de totes les aplicacions actives i de les
-- seves dades principals, en ordre descendent pel nmero total de descrregues.
------------------------------------------------------------------------------------
  PROCEDURE Aplicacions_Actives(
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      -------  Obertura del cursor sobre el select de la consulta -----------------
      OPEN p_cursor FOR 
        SELECT ap.idaplicacio, ap.versio,  count(*) as numDescarregues, ap.datapujada, ap.resoluciomin, ap.url
        FROM aplicacio ap JOIN descarrega de ON ap.idaplicacio=de.aplicacio
        WHERE ap.activa=1
        GROUP BY ap.idaplicacio, ap.versio, ap.datapujada, ap.url, ap.resoluciomin
        ORDER BY numDescarregues DESC;
                  
      rsp:='OK'; 
    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Cap parmetre d''entrada');  
    rsp:='Llista de les dades principals d''aplicacions activas.   RSP:' || rsp;
    Util.InsertarLog('Aplicacions_Actives', v_log, rsp); 
    

  END Aplicacions_Actives;
  
------------------------------------------------------------------------------------ 
--     CONSULTA DELS PASOS ON S'HA DESCARREGAT UNA APLICACIO EN UN ANY CONCRET
-- Retorna un cursor apuntant a la llista de pasos on s'ha descarregat l'aplicaci 
-- l'any donat, aix com el nmero de descrregues que ha tingut a cada pas.
------------------------------------------------------------------------------------  
  PROCEDURE Paisos_Descarrega(
    p_aplicacio IN aplicacio.idaplicacio%TYPE,
    p_any IN NUMBER,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      -------  Obertura del cursor sobre el select de la consulta -----------------
      OPEN p_cursor FOR 
       SELECT pa.nom, count(*) as numDescarregues
        FROM (aplicacio ap JOIN descarrega de ON ap.idaplicacio=de.aplicacio)
            JOIN pais pa ON de.pais=pa.idpais
        WHERE EXTRACT(YEAR FROM de.datadescarrega)=p_any AND ap.idaplicacio=p_aplicacio 
        GROUP BY pa.nom;
                  
      rsp:='OK'; 
    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Aplicacio=' || p_aplicacio ||', Any=' || p_any );  
    rsp:='Llista de pasos on s''ha descarregat l''aplicacio en l''any donat.   RSP:' || rsp;
    Util.InsertarLog('Paisos_Descarrega', v_log, rsp); 
    

  END Paisos_Descarrega;    

------------------------------------------------------------------------------------ 
--     CONSULTA DE LA ACTIVITAT DE DESCRREGUES D'UN USUARI
-- Retorna un cursor apuntant a la llista de dades de descrregues efectuades 
-- per l'usuari propietari del nmero mbil passat com a parmetre. Inclou tots
-- els dispositius del usuari amb aquest nmero mbil.
------------------------------------------------------------------------------------ 
  PROCEDURE Activitat_Usuari(
    p_telefon IN usuari.numeromobil%TYPE,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2) IS
    
  BEGIN
    BEGIN
      -------  Obertura del cursor sobre el select de la consulta -----------------
      OPEN p_cursor FOR 
        SELECT aplicacio, datadescarrega, imei, preu, modepagament
        FROM (descarrega NATURAL JOIN dispositiu) 
        WHERE imei IN
            (SELECT di.imei 
             FROM dispositiu di JOIN usuari us ON di.idpropietari=us.idusuari
             WHERE us.numeroMobil=p_telefon)
        ORDER BY datadescarrega;
                  
      rsp:='OK'; 
    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Nmero Mbil=' || p_telefon);  
    rsp:='Descrregues de l''usuari amb mbil: ' || p_telefon || '.  RSP:' || rsp;
    Util.InsertarLog('Activitat_Usuari', v_log, rsp);   
  
  
  END Activitat_Usuari;
  
------------------------------------------------------------------------------------ 
--        CONSULTA DELS USUARIS QUE MES DINERS S'HAN GASTAT EN APLICACIONS
-- Retorna un cursor apuntant a la llista dels 20 usuaris que mes diners s'han gastat
-- en aplicacions en l'any donat com a parmetre d'entrada.
------------------------------------------------------------------------------------   
  PROCEDURE Usuaris_DinersGastats(
    p_any IN NUMBER,
    p_cursor OUT NOCOPY t_RefCur,
    rsp OUT NOCOPY VARCHAR2) IS

  BEGIN
    BEGIN
      -------  Obertura del cursor sobre el select de la consulta -----------------
      OPEN p_cursor FOR 
        SELECT nom, cognoms, diners_gastats FROM
          (SELECT  idpersona, nom, cognoms, sum(preu) as diners_gastats
           FROM  (descarrega NATURAL JOIN dispositiu) JOIN persona ON idpropietari=idpersona
           WHERE EXTRACT(YEAR FROM datadescarrega)=p_any
           GROUP BY idpersona, nom, cognoms
           ORDER BY diners_gastats DESC)
        WHERE ROWNUM < 21;
                  
      rsp:='OK'; 
    EXCEPTION
      WHEN OTHERS THEN 
          rsp:='ERROR:'||SQLERRM;
    END;

    -- Inserci de control a la taula LosgsProcediments
    -------------------------------------------------------
    v_log:=('Any=' || p_any );  
    rsp:='Llista dels 20 usuaris que mes diners han gastat en l''any donat.   RSP:' || rsp;
    Util.InsertarLog('Usuaris_DinersGastats', v_log, rsp); 

    
  END Usuaris_DinersGastats;

END Consultes;
/

--------------------------------------------------------------------------------
--                TRIGGER DEL MDUL ESTADISTIC.  Consultes 1 i 2
--  S'encarrega de donar resposta a les consultes del nmero total de 
--  descrregues a la plataforma i dels euros generats fins al moment.
--------------------------------------------------------------------------------

CREATE OR REPLACE TRIGGER Estad_1_2_Descarregues_Euros
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

DECLARE
  v_usuaris NUMBER;
  v_aplicacions NUMBER;
  
BEGIN
  
  IF INSERTING THEN
      UPDATE ESTADISTICA SET valor=valor+1 
      WHERE codiestadistica=1;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, valor) VALUES (1,1);
      END IF;
    
      UPDATE ESTADISTICA SET valor=valor+:new.preu 
      WHERE codiestadistica=2;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, valor) VALUES (2,:new.preu);
      END IF;

  ELSIF UPDATING THEN
      UPDATE ESTADISTICA SET valor=valor+:new.preu-:old.preu 
      WHERE codiestadistica=2;    

  ELSIF DELETING THEN
      UPDATE ESTADISTICA SET valor=valor-1
      WHERE codiestadistica=1; 
      UPDATE ESTADISTICA SET valor=valor-:old.preu 
      WHERE codiestadistica=2;
      
  END IF;

END Estad_1_2_Descarregues_Euros; 
/

--------------------------------------------------------------------------------
--                MDUL ESTADISTIC.  Consulta 3
--  S'encarrega de donar resposta a la consulta del nmero mig d'aplicacions
--  descarregades per usuari en un any donat.
--  Consta de dos Triggers: Un a nivell de fila, que s'encarrega d'emmagatzemar
--  les dades :new i :old que despres utilitzarem per la consulta. Es important
--  fer-ho aix per que en cas contrari, es produirien errors de 'Taules Mutants'
--  al intentar llegir o modificar una d'aquestes taules dins del cos del 
--  disparador (en el nostre cas la taula DESCARREGA).
--  L'altra disparador es a nivell de ordre (statement) i utilitza les variables
--  assignades anteriorment, les quals estan definides com a variables globals
--  dins del paquet 'UTIL'.
--------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER Estad_3_mitja_apps_row
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

BEGIN
    
    util.v_imeiNou:=:new.imei;
    util.v_imeiVell:=:old.imei;
    util.v_anydescarregaNova:=EXTRACT(YEAR FROM :new.datadescarrega);
    util.v_anydescarregaVella:=EXTRACT(YEAR FROM :old.datadescarrega);

END Estad_3_mitja_apps_row; 
/


CREATE OR REPLACE TRIGGER Estad_3_mitja_apps_statement
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA


DECLARE
  v_usuaris NUMBER;
  v_aplicacions NUMBER;
  
  PROCEDURE seleccio_anyDescarregaNou IS
  BEGIN
      SELECT COUNT(DISTINCT idUsuari), COUNT(*)  INTO v_usuaris, v_aplicacions
      FROM (descarrega NATURAL JOIN dispositiu) JOIN usuari ON idPropietari=idUsuari
      WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaNova;
   
      UPDATE ESTADISTICA SET valor=v_aplicacions/v_usuaris
      WHERE codiestadistica=3 AND anyest=util.v_anydescarregaNova;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, anyest, valor) 
        VALUES (3, util.v_anydescarregaNova, v_aplicacions/v_usuaris);
      END IF;
  END seleccio_anyDescarregaNou;
  
  PROCEDURE seleccio_anyDescarregaVell IS
  BEGIN
      SELECT COUNT(DISTINCT idUsuari), COUNT(*)  INTO v_usuaris, v_aplicacions
      FROM (descarrega NATURAL JOIN dispositiu) JOIN usuari ON idPropietari=idUsuari
      WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaVella;
   
        -- Es controla si quedan usuaris que hagin baixat aplicacions aquest any. Si no ni ha, es borra el registre
        -- corresponent de la taula Estadstica.
      IF v_usuaris > 0 THEN
        UPDATE ESTADISTICA SET valor=v_aplicacions/v_usuaris
        WHERE codiestadistica=3 AND anyest=util.v_anydescarregaVella;
      ELSE
        DELETE FROM ESTADISTICA
        WHERE codiestadistica=3 AND anyest=util.v_anydescarregaVella;
      END IF;
  END seleccio_anyDescarregaVell;
  
BEGIN
  
  IF INSERTING THEN
      seleccio_anyDescarregaNou;

  ELSIF UPDATING THEN
  -- Es controla si s'han canviat els valors de 'Imei' o l'any de descrrega, que son els que poden
  -- afectar a la mitja d'aplicacions descarregades per usuari en un any donat.
      IF util.v_imeiNou <> util.v_imeiVell OR util.v_anydescarregaNova <> util.v_anydescarregaVella THEN 
        seleccio_anyDescarregaNou;
        seleccio_anyDescarregaVell;
      END IF;

  ELSIF DELETING THEN
      seleccio_anyDescarregaVell;
 
  END IF;

END Estad_3_mitja_apps_statement; 
/

--------------------------------------------------------------------------------
--                  MDUL ESTADISTIC.  Consulta 4
--  S'encarrega de donar resposta a la consulta del desenvolupador que ha tingut
--  el mxim numero de descarregues de les seves aplicacions en un any donat.
--  Consta de dos Triggers: Un a nivell de fila, que s'encarrega d'emmagatzemar
--  les dades :new i :old que despres utilitzarem per la consulta. Es important
--  fer-ho aix per que en cas contrari, es produirien errors de 'Taules Mutants'
--  al intentar llegir o modificar una d'aquestes taules dins del cos del 
--  disparador (en el nostre cas la taula DESCARREGA).
--  L'altra disparador es a nivell de ordre (statement) i utilitza les variables
--  assignades anteriorment, les quals estan definides com a variables globals
--  dins del paquet 'UTIL'.
--------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER Estad_4_desc_desenv_row
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

BEGIN
    
    util.v_aplicacioNova:=:new.aplicacio;
    util.v_aplicacioVella:=:old.aplicacio;
    util.v_anydescarregaNova:=EXTRACT(YEAR FROM :new.datadescarrega);
    util.v_anydescarregaVella:=EXTRACT(YEAR FROM :old.datadescarrega);

END Estad_4_desc_desenv_row; 
/


CREATE OR REPLACE TRIGGER Estad_4_desc_desenv_statement
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA


DECLARE
  v_desenvolupador desenvolupador.iddesenvolupador%TYPE;
  v_aplicacions NUMBER;
  
  PROCEDURE seleccio_anyDescarregaNou IS
  BEGIN
      SELECT desenvolupador, apps_descarregades INTO v_desenvolupador, v_aplicacions FROM
        (SELECT  apl.desenvolupador, count(*) as apps_descarregades
         FROM (descarrega de JOIN aplicacio ap ON de.aplicacio=ap.idaplicacio) 
             JOIN apldesenvolupada apl ON apl.aplicacio=ap.idaplicacio
         WHERE EXTRACT(YEAR FROM de.datadescarrega)=util.v_anydescarregaNova
         GROUP BY apl.desenvolupador
         ORDER BY apps_descarregades DESC, apl.desenvolupador ASC)
      WHERE ROWNUM=1;
   
      UPDATE ESTADISTICA SET valor=v_aplicacions, desenvolupador=v_desenvolupador
      WHERE codiestadistica=4 AND anyest=util.v_anydescarregaNova;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, anyest, valor, desenvolupador) 
        VALUES (4, util.v_anydescarregaNova, v_aplicacions, v_desenvolupador);
      END IF;
  END seleccio_anyDescarregaNou;
  
  PROCEDURE seleccio_anyDescarregaVell IS
  BEGIN
      SELECT desenvolupador, apps_descarregades INTO v_desenvolupador, v_aplicacions FROM
        (SELECT  apl.desenvolupador, count(*) as apps_descarregades
         FROM (descarrega de JOIN aplicacio ap ON de.aplicacio=ap.idaplicacio) 
             JOIN apldesenvolupada apl ON apl.aplicacio=ap.idaplicacio
         WHERE EXTRACT(YEAR FROM de.datadescarrega)=util.v_anydescarregaVella
         GROUP BY apl.desenvolupador
         ORDER BY apps_descarregades DESC, apl.desenvolupador ASC)
      WHERE ROWNUM=1;
   
      UPDATE ESTADISTICA SET valor=v_aplicacions, desenvolupador=v_desenvolupador
      WHERE codiestadistica=4 AND anyest=util.v_anydescarregaVella;
              
  EXCEPTION
    -- Es controla si quedan aplicacions que s'hagin descarregat aquest any. Si no ni ha, es borra el registre
    -- corresponent de la taula Estadstica.
    WHEN NO_DATA_FOUND THEN
      DELETE FROM ESTADISTICA
      WHERE codiestadistica=4 AND anyest=util.v_anydescarregaVella;

  END seleccio_anyDescarregaVell;
  
BEGIN
  
  IF INSERTING THEN
      seleccio_anyDescarregaNou;

  ELSIF UPDATING THEN
  -- Es controla si s'han canviat els valors de 'Aplicacio' o l'any de descrrega, que son els que poden
  -- afectar al desenvolupador que te el mxim numero de descrregues en un any donat.
      IF util.v_aplicacioNova <> util.v_aplicacioVella OR util.v_anydescarregaNova <> util.v_anydescarregaVella THEN 
        seleccio_anyDescarregaNou;
        seleccio_anyDescarregaVell;
      END IF;

  ELSIF DELETING THEN
      seleccio_anyDescarregaVell;
 
  END IF;

END Estad_4_desc_desenv_statement; 
/

--------------------------------------------------------------------------------
--                  MDUL ESTADISTIC.  Consulta 5
--  S'encarrega de donar resposta a la consulta de l'aplicacio que mes diners
--  ha recaudat en descarregues, aixi com el seu desenvolupador, en un any donat.
--  Consta de dos Triggers: Un a nivell de fila, que s'encarrega d'emmagatzemar
--  les dades :new i :old que despres utilitzarem per la consulta. Es important
--  fer-ho aix per que en cas contrari, es produirien errors de 'Taules Mutants'
--  al intentar llegir o modificar una d'aquestes taules dins del cos del 
--  disparador (en el nostre cas la taula DESCARREGA).
--  L'altra disparador es a nivell de ordre (statement) i utilitza les variables
--  assignades anteriorment, les quals estan definides com a variables globals
--  dins del paquet 'UTIL'.
--------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER Estad_5_app_maxRecaudacio_row
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

BEGIN
    
    util.v_aplicacioNova:=:new.aplicacio;
    util.v_aplicacioVella:=:old.aplicacio;
    util.v_anydescarregaNova:=EXTRACT(YEAR FROM :new.datadescarrega);
    util.v_anydescarregaVella:=EXTRACT(YEAR FROM :old.datadescarrega);

END Estad_5_app_maxRecaudacio_row; 
/


CREATE OR REPLACE TRIGGER Estad_5_app_maxRecaudacio_stat
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA

DECLARE
  v_desenvolupador estadistica.desenvolupador%TYPE;
  v_aplicacio aplicacio.idaplicacio%TYPE;
  v_total_desenvolupadors NUMBER;
  
  PROCEDURE seleccio_anyDescarregaNou IS
  BEGIN
      SELECT aplicacio INTO v_aplicacio FROM DESCARREGA
         WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaNova
         GROUP BY aplicacio
         HAVING sum(preu)=
            (SELECT max(sum(preu)) 
             FROM descarrega 
             WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaNova
             GROUP BY aplicacio );
             
      SELECT desenvolupador INTO v_desenvolupador FROM
         (SELECT desenvolupador FROM APLDESENVOLUPADA
          WHERE aplicacio=v_aplicacio)
      WHERE ROWNUM=1;
      
      SELECT count(*) INTO v_total_desenvolupadors FROM APLDESENVOLUPADA
          WHERE aplicacio=v_aplicacio;
      
      v_desenvolupador:=v_desenvolupador || ' , i ' || TO_CHAR(v_total_desenvolupadors-1) || ' mes.';
      UPDATE ESTADISTICA SET  desenvolupador=v_desenvolupador, aplicacio=v_aplicacio
      WHERE codiestadistica=5 AND anyest=util.v_anydescarregaNova;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, anyest, desenvolupador, aplicacio) 
        VALUES (5, util.v_anydescarregaNova, v_desenvolupador, v_aplicacio);
      END IF;
  END seleccio_anyDescarregaNou;
  
  PROCEDURE seleccio_anyDescarregaVell IS
  BEGIN
     SELECT aplicacio INTO v_aplicacio FROM DESCARREGA
         WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaVella
         GROUP BY aplicacio
         HAVING sum(preu)=
            (SELECT max(sum(preu)) 
             FROM descarrega 
             WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaVella
             GROUP BY aplicacio );
             
      SELECT desenvolupador INTO v_desenvolupador FROM
         (SELECT desenvolupador FROM APLDESENVOLUPADA
          WHERE aplicacio=v_aplicacio)
      WHERE ROWNUM=1;
      
      SELECT count(*) INTO v_total_desenvolupadors FROM APLDESENVOLUPADA
          WHERE aplicacio=v_aplicacio;
   
      v_desenvolupador:=v_desenvolupador || ' , i ' || TO_CHAR(v_total_desenvolupadors-1) || ' mes.';
      UPDATE ESTADISTICA SET  desenvolupador=v_desenvolupador, aplicacio=v_aplicacio
      WHERE codiestadistica=5 AND anyest=util.v_anydescarregaVella;
              
  EXCEPTION
    -- Es controla si quedan aplicacions que s'hagin descarregat aquest any. Si no ni ha, es borra el registre
    -- corresponent de la taula Estadstica.
    WHEN NO_DATA_FOUND THEN
      DELETE FROM ESTADISTICA
      WHERE codiestadistica=5 AND anyest=util.v_anydescarregaVella;

  END seleccio_anyDescarregaVell;
  
BEGIN
  
  IF INSERTING THEN
      seleccio_anyDescarregaNou;

  ELSIF UPDATING THEN
  -- Es controla si s'han canviat els valors de 'Aplicacio' o l'any de descrrega, que son els que poden
  -- afectar a l'aplicaci que mes diners ha recaudat en descarregues en un any donat.
      IF util.v_aplicacioNova <> util.v_aplicacioVella OR util.v_anydescarregaNova <> util.v_anydescarregaVella THEN 
        seleccio_anyDescarregaNou;
        seleccio_anyDescarregaVell;
      END IF;

  ELSIF DELETING THEN
      seleccio_anyDescarregaVell;
 
  END IF;

END Estad_5_app_maxRecaudacio_stat; 
/

--------------------------------------------------------------------------------
--                  MDUL ESTADISTIC.  Consulta 6
--  S'encarrega de donar resposta a la consulta del numero d'usuaris diferents
--  que han fet com a minim una descrrega en un pais i un any determinat.
--  Consta de dos Triggers: Un a nivell de fila, que s'encarrega d'emmagatzemar
--  les dades :new i :old que despres utilitzarem per la consulta. Es important
--  fer-ho aix per que en cas contrari, es produirien errors de 'Taules Mutants'
--  al intentar llegir o modificar una d'aquestes taules dins del cos del 
--  disparador (en el nostre cas la taula DESCARREGA).
--  L'altra disparador es a nivell de ordre (statement) i utilitza les variables
--  assignades anteriorment, les quals estan definides com a variables globals
--  dins del paquet 'UTIL'.
--------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER Estad_6_Num_Usuaris_row
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

BEGIN
    
    util.v_paisNou:=:new.pais;
    util.v_paisVell:=:old.pais;
    util.v_imeiNou:=:new.imei;
    util.v_imeiVell:=:old.imei;
    util.v_anydescarregaNova:=EXTRACT(YEAR FROM :new.datadescarrega);
    util.v_anydescarregaVella:=EXTRACT(YEAR FROM :old.datadescarrega);

END Estad_6_Num_Usuaris_row; 
/


CREATE OR REPLACE TRIGGER Estad_6_Num_Usuaris_stat
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA

DECLARE

  v_total_usuaris NUMBER;
  
  PROCEDURE seleccio_anyDescarregaNou IS
  BEGIN
    SELECT count(*) INTO v_total_usuaris FROM
      (SELECT us.idusuari 
       FROM (DESCARREGA NATURAL JOIN DISPOSITIU) JOIN USUARI us ON idPropietari=idUsuari
          WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaNova AND pais =util.v_paisNou
          GROUP BY us.idusuari);
  
      UPDATE ESTADISTICA SET  valor=v_total_usuaris
      WHERE codiestadistica=6 AND anyest=util.v_anydescarregaNova AND pais=util.v_paisNou;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, anyest, pais, valor) 
        VALUES (6, util.v_anydescarregaNova, util.v_paisNou, v_total_usuaris);
      END IF;
      
  END seleccio_anyDescarregaNou;
  
  PROCEDURE seleccio_anyDescarregaVell IS
  BEGIN
    SELECT count(*) INTO v_total_usuaris FROM
      (SELECT us.idusuari 
       FROM (DESCARREGA NATURAL JOIN DISPOSITIU) JOIN USUARI us ON idPropietari=idUsuari
          WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaVella AND pais =util.v_paisVell
          GROUP BY us.idusuari);

    -- Es controla si quedan usuaris del pais donat que s'hagin descarregat aplicacions aquest any. 
    -- Si no ni ha, es borra el registre corresponent de la taula Estadstica.
      IF v_total_usuaris=0 THEN
        DELETE FROM ESTADISTICA
        WHERE codiestadistica=6 AND anyest=util.v_anydescarregaVella AND pais=util.v_paisVell;
      ELSE
        UPDATE ESTADISTICA SET valor=v_total_usuaris
        WHERE codiestadistica=6 AND anyest=util.v_anydescarregaVella AND pais=util.v_paisVell;
      END IF;
      
  END seleccio_anyDescarregaVell;
  
BEGIN
  
  IF INSERTING THEN
      seleccio_anyDescarregaNou;

  ELSIF UPDATING THEN
  -- Es controla si s'han canviat els valors del Pais, l'any  o l'Imei de descrrega, que son els que poden
  -- afectar al nmero d'usuaris diferents que han fet al menys una descrrega.
      IF util.v_paisNou <> util.v_paisVell OR util.v_anydescarregaNova <> util.v_anydescarregaVella 
                                           OR util.v_imeiNou <> util.v_imeiVell THEN 
        seleccio_anyDescarregaNou;
        seleccio_anyDescarregaVell;
      END IF;

  ELSIF DELETING THEN
      seleccio_anyDescarregaVell;
 
  END IF;

END Estad_6_Num_Usuaris_stat; 
/

--------------------------------------------------------------------------------
--                  MDUL ESTADISTIC.  Consulta 7
--  S'encarrega de donar resposta a la consulta dels ingresos totals que han
--  generat els usuaris de un pais en un any determinat.
--  Consta de dos Triggers: Un a nivell de fila, que s'encarrega d'emmagatzemar
--  les dades :new i :old que despres utilitzarem per la consulta. Es important
--  fer-ho aix per que en cas contrari, es produirien errors de 'Taules Mutants'
--  al intentar llegir o modificar una d'aquestes taules dins del cos del 
--  disparador (en el nostre cas la taula DESCARREGA).
--  L'altra disparador es a nivell de ordre (statement) i utilitza les variables
--  assignades anteriorment, les quals estan definides com a variables globals
--  dins del paquet 'UTIL'.
--------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER Estad_7_Ing_Usuaris_Pais_row
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

BEGIN
    
    util.v_paisNou:=:new.pais;
    util.v_paisVell:=:old.pais;
    util.v_anydescarregaNova:=EXTRACT(YEAR FROM :new.datadescarrega);
    util.v_anydescarregaVella:=EXTRACT(YEAR FROM :old.datadescarrega);
    util.v_preuNou:=:new.preu;
    util.v_preuVell:=:old.preu;

END Estad_7_Ing_Usuaris_Pais_row; 
/


CREATE OR REPLACE TRIGGER Estad_7_Ing_Usuaris_Pais_stat
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA

DECLARE

  v_total_ingresos NUMBER;
  v_num_descarregues NUMBER;
  
  PROCEDURE seleccio_anyDescarregaNou IS
  BEGIN
     SELECT sum(preu) INTO v_total_ingresos FROM
       (SELECT * FROM descarrega
        WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaNova AND pais = util.v_paisNou);
  
      UPDATE ESTADISTICA SET  valor=v_total_ingresos
      WHERE codiestadistica=7 AND anyest=util.v_anydescarregaNova AND pais=util.v_paisNou;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, anyest, pais, valor) 
        VALUES (7, util.v_anydescarregaNova, util.v_paisNou, v_total_ingresos);
      END IF;
      
  END seleccio_anyDescarregaNou;
  
  PROCEDURE seleccio_anyDescarregaVell IS
  BEGIN
     SELECT sum(preu), count(*) INTO v_total_ingresos, v_num_descarregues FROM
       (SELECT preu FROM descarrega
        WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaVella AND pais=util.v_paisVell);

    -- Es controla si quedan descrregues del pais i any donats. 
    -- Si no ni ha, es borra el registre corresponent de la taula Estadstica.
      IF v_num_descarregues=0 THEN
        DELETE FROM ESTADISTICA
        WHERE codiestadistica=7 AND anyest=util.v_anydescarregaVella AND pais=util.v_paisVell;
      ELSE
        UPDATE ESTADISTICA SET valor=v_total_ingresos
        WHERE codiestadistica=7 AND anyest=util.v_anydescarregaVella AND pais=util.v_paisVell;
      END IF;
      
  END seleccio_anyDescarregaVell;
  
BEGIN
  
  IF INSERTING THEN
      seleccio_anyDescarregaNou;

  ELSIF UPDATING THEN
  -- Es controla si s'han canviat els valors del Pais, l'any o el preu , que son els que poden
  -- afectar als ingresos totals.
      IF util.v_paisNou <> util.v_paisVell OR util.v_anydescarregaNova <> util.v_anydescarregaVella 
            OR util.v_preuNou <> util.v_preuVell THEN 
        seleccio_anyDescarregaNou;
        seleccio_anyDescarregaVell;
      END IF;

  ELSIF DELETING THEN
      seleccio_anyDescarregaVell;
 
  END IF;

END Estad_7_Ing_Usuaris_Pais_stat; 
/


--------------------------------------------------------------------------------
--                  MDUL ESTADISTIC.  Consulta 8
--  S'encarrega de donar resposta a la consulta del nmero d'aplicacions
--  diferents descarregades al menys una vegada en un pais i un any determinat.
--  Consta de dos Triggers: Un a nivell de fila, que s'encarrega d'emmagatzemar
--  les dades :new i :old que despres utilitzarem per la consulta. Es important
--  fer-ho aix per que en cas contrari, es produirien errors de 'Taules Mutants'
--  al intentar llegir o modificar una d'aquestes taules dins del cos del 
--  disparador (en el nostre cas la taula DESCARREGA).
--  L'altra disparador es a nivell de ordre (statement) i utilitza les variables
--  assignades anteriorment, les quals estan definides com a variables globals
--  dins del paquet 'UTIL'.
--------------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER Estad_8_Apps_Diferents_row
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA
FOR EACH ROW

BEGIN
    
    util.v_paisNou:=:new.pais;
    util.v_paisVell:=:old.pais;
    util.v_anydescarregaNova:=EXTRACT(YEAR FROM :new.datadescarrega);
    util.v_anydescarregaVella:=EXTRACT(YEAR FROM :old.datadescarrega);
    util.v_aplicacioNova:=:new.aplicacio;
    util.v_aplicacioVella:=:old.aplicacio;

END Estad_8_Apps_Diferents_row; 
/


CREATE OR REPLACE TRIGGER Estad_8_Apps_Diferents_stat
AFTER INSERT OR DELETE OR UPDATE ON DESCARREGA

DECLARE

  v_total_aplicacions NUMBER;
  
  PROCEDURE seleccio_anyDescarregaNou IS
  BEGIN
      SELECT count(*) INTO v_total_aplicacions FROM
        (SELECT aplicacio FROM descarrega
         WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaNova AND pais = util.v_paisNou
         GROUP BY aplicacio);
  
      UPDATE ESTADISTICA SET  valor=v_total_aplicacions
      WHERE codiestadistica=8 AND anyest=util.v_anydescarregaNova AND pais=util.v_paisNou;
      IF SQL%NOTFOUND THEN
        INSERT INTO ESTADISTICA (codiestadistica, anyest, pais, valor) 
        VALUES (8, util.v_anydescarregaNova, util.v_paisNou, v_total_aplicacions);
      END IF;
      
  END seleccio_anyDescarregaNou;
  
  PROCEDURE seleccio_anyDescarregaVell IS
  BEGIN
      SELECT count(*) INTO v_total_aplicacions FROM
        (SELECT aplicacio FROM descarrega
         WHERE EXTRACT(YEAR FROM datadescarrega)=util.v_anydescarregaVella AND pais = util.v_paisVell
         GROUP BY aplicacio);

    -- Es controla si quedan aplicacions descarregades del pais i any donats. 
    -- Si no ni ha, es borra el registre corresponent de la taula Estadstica.
      IF v_total_aplicacions=0 THEN
        DELETE FROM ESTADISTICA
        WHERE codiestadistica=8 AND anyest=util.v_anydescarregaVella AND pais=util.v_paisVell;
      ELSE
        UPDATE ESTADISTICA SET valor=v_total_aplicacions
        WHERE codiestadistica=8 AND anyest=util.v_anydescarregaVella AND pais=util.v_paisVell;
      END IF;
      
  END seleccio_anyDescarregaVell;
  
BEGIN
  
  IF INSERTING THEN
      seleccio_anyDescarregaNou;

  ELSIF UPDATING THEN
  -- Es controla si s'han canviat els valors del Pais, l'any o l'aplicaci, que son els que poden
  -- afectar al nmero d'aplicacions descarregades diferents al menys una vegada.
      IF util.v_paisNou <> util.v_paisVell OR util.v_anydescarregaNova <> util.v_anydescarregaVella 
            OR util.v_aplicacioNova <> util.v_aplicacioVella THEN 
        seleccio_anyDescarregaNou;
        seleccio_anyDescarregaVell;
      END IF;

  ELSIF DELETING THEN
      seleccio_anyDescarregaVell;
 
  END IF;

END Estad_8_Apps_Diferents_stat; 
/

